@@ -13,16 +13,19 @@
  * a copy of the license along with this program.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <string.h>
 #include <time.h>
 #include <math.h> /* for sqrt() */
 #include <setjmp.h>
 #include <errno.h>
 #include <float.h>
-#include <glib.h>
+#include "uf_glib.h"
 #include <glib/gi18n.h> /*For _(String) definition - NKBJ*/
+#include <sys/types.h>
 #include "dcraw_api.h"
 #include "dcraw.h"
 
@@ -48,9 +51,8 @@ void vng_interpolate_INDI(gushort (*image)[4], const unsigned filters,
 void ahd_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, float rgb_cam[3][4],
     void *dcraw);
-void eahd_interpolate_INDI(gushort (*image)[4], const unsigned filters,
-    const int width, const int height, const int colors, float rgb_cam[3][4],
-    void *dcraw);
+void color_smooth(gushort (*image)[4], const int width, const int height,
+    const int passes);
 void ppg_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, void *dcraw);
 void flip_image_INDI(gushort (*image)[4], int *height_p, int *width_p,
@@ -68,18 +70,18 @@ int dcraw_open(dcraw_data *h,char *filename)
     d->verbose = 1;
     d->ifname = g_strdup(filename);
     if (setjmp(d->failure)) {
-        d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
-        h->message = d->messageBuffer;
+	d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
+	h->message = d->messageBuffer;
 	delete d;
-        return DCRAW_ERROR;
+	return DCRAW_ERROR;
     }
-    if (!(d->ifp = fopen (d->ifname, "rb"))) {
-        d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot open file %s: %s\n"),
-                filename, strerror(errno));
-        g_free(d->ifname);
-        h->message = d->messageBuffer;
+    if (!(d->ifp = g_fopen (d->ifname, "rb"))) {
+	d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot open file %s: %s\n"),
+		filename, strerror(errno));
+	g_free(d->ifname);
+	h->message = d->messageBuffer;
 	delete d;
-        return DCRAW_OPEN_ERROR;
+	return DCRAW_OPEN_ERROR;
     }
     d->identify();
     /* We first check if dcraw recognizes the file, this is equivalent
@@ -87,22 +89,22 @@ int dcraw_open(dcraw_data *h,char *filename)
     if (!d->make[0]) {
 	d->dcraw_message(DCRAW_OPEN_ERROR,_("%s: unsupported file format.\n"),
 		d->ifname);
-        fclose(d->ifp);
-        g_free(d->ifname);
-        h->message = d->messageBuffer;
+	fclose(d->ifp);
+	g_free(d->ifname);
+	h->message = d->messageBuffer;
 	int lastStatus = d->lastStatus;
 	delete d;
-        return lastStatus;
+	return lastStatus;
     }
     /* Next we check if dcraw can decode the file */
     if (!d->is_raw) {
 	d->dcraw_message(DCRAW_OPEN_ERROR,_("Cannot decode file %s\n"), d->ifname);
-        fclose(d->ifp);
-        g_free(d->ifname);
-        h->message = d->messageBuffer;
+	fclose(d->ifp);
+	g_free(d->ifname);
+	h->message = d->messageBuffer;
 	int lastStatus = d->lastStatus;
 	delete d;
-        return lastStatus;
+	return lastStatus;
     }
     if (d->load_raw == &DCRaw::kodak_ycbcr_load_raw) {
 	d->height += d->height & 1;
@@ -124,9 +126,9 @@ int dcraw_open(dcraw_data *h,char *filename)
     h->pixel_aspect = d->pixel_aspect;
     /* copied from dcraw's main() */
     switch ((d->flip+3600) % 360) {
-        case 270: d->flip = 5; break;
-        case 180: d->flip = 3; break;
-        case  90: d->flip = 6;
+	case 270: d->flip = 5; break;
+	case 180: d->flip = 3; break;
+	case  90: d->flip = 6;
     }
     h->flip = d->flip;
     h->toneCurveSize = d->tone_curve_size;
@@ -149,23 +151,23 @@ int dcraw_open(dcraw_data *h,char *filename)
 int dcraw_image_dimensions(dcraw_data *raw, int flip, int *height, int *width)
 {
     if (raw->fuji_width) {
-        /* Copied from DCRaw's fuji_rotate() */
-        *width = (int)(raw->fuji_width / raw->fuji_step);
-        *height = (int)((raw->height - raw->fuji_width) / raw->fuji_step);
+	/* Copied from DCRaw's fuji_rotate() */
+	*width = (int)((raw->fuji_width - 1) / raw->fuji_step);
+	*height = (int)((raw->height - raw->fuji_width + 1) / raw->fuji_step);
     } else {
-        if (raw->pixel_aspect < 1)
-            *height = (int)(raw->height / raw->pixel_aspect + 0.5);
-        else
-            *height = raw->height;
-        if (raw->pixel_aspect > 1)
-            *width = (int)(raw->width * raw->pixel_aspect + 0.5);
-        else
-            *width = raw->width;
+	if (raw->pixel_aspect < 1)
+	    *height = (int)(raw->height / raw->pixel_aspect + 0.5);
+	else
+	    *height = raw->height;
+	if (raw->pixel_aspect > 1)
+	    *width = (int)(raw->width * raw->pixel_aspect + 0.5);
+	else
+	    *width = raw->width;
     }
     if (flip & 4) {
-        int tmp = *height;
-        *height = *width;
-        *width = tmp;
+	int tmp = *height;
+	*height = *width;
+	*width = tmp;
     }
     return DCRAW_SUCCESS;
 }
@@ -180,10 +182,10 @@ int dcraw_load_raw(dcraw_data *h)
     d->messageBuffer = NULL;
     d->lastStatus = DCRAW_SUCCESS;
     if (setjmp(d->failure)) {
-        d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
-        h->message = d->messageBuffer;
+	d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
+	h->message = d->messageBuffer;
 	delete d;
-        return DCRAW_ERROR;
+	return DCRAW_ERROR;
     }
     h->raw.height = d->iheight = (h->height+h->shrink) >> h->shrink;
     h->raw.width = d->iwidth = (h->width+h->shrink) >> h->shrink;
@@ -192,21 +194,21 @@ int dcraw_load_raw(dcraw_data *h)
     d->meta_data = (char *) (d->image + d->iheight*d->iwidth);
     /* copied from the end of dcraw's identify() */
     if (d->filters && d->colors == 3) {
-        for (i=0; i < 32; i+=4) {
-            if ((d->filters >> i & 15) == 9) d->filters |= 2 << i;
-            if ((d->filters >> i & 15) == 6) d->filters |= 8 << i;
-        }
-        d->colors++;
+	for (i=0; i < 32; i+=4) {
+	    if ((d->filters >> i & 15) == 9) d->filters |= 2 << i;
+	    if ((d->filters >> i & 15) == 6) d->filters |= 8 << i;
+	}
+	d->colors++;
     }
     h->raw.colors = d->colors;
     h->fourColorFilters = d->filters;
     d->dcraw_message(DCRAW_VERBOSE,_("Loading %s %s image from %s ...\n"),
-                d->make, d->model, d->ifname);
+	    d->make, d->model, d->ifname);
     fseek (d->ifp, d->data_offset, SEEK_SET);
     (d->*d->load_raw)();
     d->bad_pixels();
     if (d->is_foveon) {
-        d->foveon_interpolate();
+	d->foveon_interpolate();
 	h->raw.width = h->width = d->width;
 	h->raw.height = h->height = d->height;
     }
@@ -253,6 +255,8 @@ int dcraw_load_thumb(dcraw_data *h, dcraw_image_data *thumb)
 	h->thumbType = jpeg_thumb_type;
     } else if (d->write_thumb==&DCRaw::ppm_thumb) {
 	h->thumbType = ppm_thumb_type;
+	// Copied from dcraw's ppm_thumb()
+	h->thumbBufferLength = thumb->width*thumb->height*3;
     } else {
 	dcraw_message(d, DCRAW_ERROR,
 		_("Unsupported thumb format for %s"), d->ifname);
@@ -261,63 +265,104 @@ int dcraw_load_thumb(dcraw_data *h, dcraw_image_data *thumb)
     return d->lastStatus;
 }
 
-int dcraw_finalize_shrink(dcraw_image_data *f, dcraw_data *hh, int scale)
+/* Grab a pixel from the raw data, doing dark frame removal on the fly.
+ *
+ * The most obvious algorithm for dark frame removal is to simply
+ * subtract the dark frame from the image (rounding negative values to
+ * zero).  However, this leaves holes in the resulting image that need
+ * to be interpolated from the surrounding pixels.
+ *
+ * The processing works by subtracting the dark frame as usual for most
+ * pixels.  For all pixels where the dark frame is brighter than a given
+ * threshold, the result is instead calculated as the average of the
+ * dark-adjusted values of the 4 surrounding pixels.  By this method,
+ * only hot pixels (as determined by the threshold) are examined and
+ * recalculated.
+ */
+static int get_dark_pixel(const dcraw_data *h, const dcraw_data *dark,
+			  int i, int cl)
+{
+    return MAX(h->raw.image[i][cl] - dark->raw.image[i][cl], 0);
+}
+
+static int get_pixel(const dcraw_data *h, const dcraw_data *dark,
+		     int r, int c, int cl, int pixels)
+{
+    int i = r * h->raw.width + c;
+    int pixel = h->raw.image[i][cl];
+    if (dark != 0) {
+	int w = h->raw.width;
+	pixel = (dark->raw.image[i][cl] <= dark->thresholds[cl])
+	    ? MAX(pixel - dark->raw.image[i][cl], 0)
+	    : (get_dark_pixel(h, dark, i + ((i >= 1) ? -1 : 1), cl) +
+	       get_dark_pixel(h, dark, i + ((i < pixels-1) ? 1 : -1), cl) +
+	       get_dark_pixel(h, dark, i + ((i >= w) ? -w : w), cl) +
+	       get_dark_pixel(h, dark, i + ((i < pixels-w) ? w : -w), cl))
+	    / 4;
+    }
+    return pixel;
+}
+
+int dcraw_finalize_shrink(dcraw_image_data *f, dcraw_data *hh,
+			  dcraw_data *dark, int scale)
 {
     DCRaw *d = (DCRaw *)hh->dcraw;
-    int h, w, fujiWidth, r, c, ri, ci, cl, norm, s, recombine;
+    int h, w, fujiWidth, r, c, ri, ci, cl, norm, s, recombine, pixels;
     int f4, sum[4], count[4];
 
     g_free(d->messageBuffer);
     d->messageBuffer = NULL;
     d->lastStatus = DCRAW_SUCCESS;
+    pixels = hh->raw.width * hh->raw.height;
 
     recombine = ( hh->colors==3 && hh->raw.colors==4 );
     f->colors = hh->colors;
 
+    int black = dark ? MAX(hh->black - dark->black, 0) : hh->black;
     /* hh->raw.image is shrunk in half if there are filters.
      * If scale is odd we need to "unshrink" it using the info in
      * hh->fourColorFilters before scaling it. */
     if (hh->filters!=0 && scale%2==1) {
-        /* I'm skiping the last row/column if it is not a full row/column */
+	/* I'm skiping the last row/column if it is not a full row/column */
 	f->height = h = hh->height / scale;
 	f->width = w = hh->width / scale;
 	fujiWidth = hh->fuji_width / scale;
-	f->image = g_new0(dcraw_image_type, h * w);
+	f->image = (dcraw_image_type *)
+		g_realloc(f->image, h * w * sizeof(dcraw_image_type));
 	f4 = hh->fourColorFilters;
 	for(r=0; r<h; r++) {
 	    for(c=0; c<w; c++) {
 		for (cl=0; cl<hh->raw.colors; cl++) sum[cl] = count[cl] = 0;
 		for (ri=0; ri<scale; ri++)
 		    for (ci=0; ci<scale; ci++) {
-			sum[fc_INDI(f4, r*scale+ri, c*scale+ci)] +=
-			    hh->raw.image
-				[(r*scale+ri)/2*hh->raw.width+(c*scale+ci)/2]
-				[fc_INDI(f4, r*scale+ri, c*scale+ci)];
-			count[fc_INDI(f4, r*scale+ri, c*scale+ci)]++;
+			cl = fc_INDI(f4, r*scale+ri, c*scale+ci);
+			sum[cl] += get_pixel(hh, dark,
+				r*scale+ri, c*scale+ci, cl, pixels);
+			count[cl]++;
 		    }
 		for (cl=0; cl<hh->raw.colors; cl++)
 		    f->image[r*w+c][cl] =
-				MAX(sum[cl]/count[cl] - hh->black,0);
+				MAX(sum[cl]/count[cl] - black,0);
 		if (recombine) f->image[r*w+c][1] =
 			(f->image[r*w+c][1] + f->image[r*w+c][3])>>1;
 	    }
 	}
     } else {
 	if (hh->filters!=0) scale /= 2;
-        /* I'm skiping the last row/column if it is not a full row/column */
+	/* I'm skiping the last row/column if it is not a full row/column */
 	f->height = h = hh->raw.height / scale;
 	f->width = w = hh->raw.width / scale;
 	fujiWidth = ( (hh->fuji_width+hh->shrink) >> hh->shrink ) / scale;
-	f->image = g_new0(dcraw_image_type, h * w);
+	f->image = (dcraw_image_type *)g_realloc(
+			f->image, h * w * sizeof(dcraw_image_type));
 	norm = scale * scale;
 	for(r=0; r<h; r++) {
 	    for(c=0; c<w; c++) {
 		for (cl=0; cl<hh->raw.colors; cl++) {
 		    for (ri=0, s=0; ri<scale; ri++)
 			for (ci=0; ci<scale; ci++)
-			    s += hh->raw.image
-				[(r*scale+ri)*hh->raw.width+c*scale+ci][cl];
-                    f->image[r*w+c][cl] = MAX(s/norm - hh->black,0);
+			    s += get_pixel(hh, dark, r*scale+ri, c*scale+ci, cl, pixels);
+		    f->image[r*w+c][cl] = MAX(s/norm - black,0);
 		}
 		if (recombine) f->image[r*w+c][1] =
 		    (f->image[r*w+c][1] + f->image[r*w+c][3])>>1;
@@ -347,31 +392,31 @@ int dcraw_image_resize(dcraw_image_data *image, int size)
     norm = div * div;
 
     for(r=0; r<image->height; r++) {
-        /* r should be divided between ri and rii */
-        ri = r * mul / div;
-        rii = (r+1) * mul / div;
-        /* with weights riw and riiw (riw+riiw==mul) */
-        riw = rii * div - r * mul;
-        riiw = (r+1) * mul - rii * div;
-        if (rii>=h) {rii=h-1; riiw=0;}
-        if (ri>=h) {ri=h-1; riw=0;}
-        for(c=0; c<image->width; c++) {
-            ci = c * mul / div;
-            cii = (c+1) * mul / div;
-            ciw = cii * div - c * mul;
-            ciiw = (c+1) * mul - cii * div;
-            if (cii>=w) {cii=w-1; ciiw=0;}
-            if (ci>=w) {ci=w-1; ciw=0;}
-            for (cl=0; cl<image->colors; cl++) {
-                iBuf[ri *w+ci ][cl] += image->image[r*wid+c][cl]*riw *ciw ;
-                iBuf[ri *w+cii][cl] += image->image[r*wid+c][cl]*riw *ciiw;
-                iBuf[rii*w+ci ][cl] += image->image[r*wid+c][cl]*riiw*ciw ;
-                iBuf[rii*w+cii][cl] += image->image[r*wid+c][cl]*riiw*ciiw;
-            }
-        }
+	/* r should be divided between ri and rii */
+	ri = r * mul / div;
+	rii = (r+1) * mul / div;
+	/* with weights riw and riiw (riw+riiw==mul) */
+	riw = rii * div - r * mul;
+	riiw = (r+1) * mul - rii * div;
+	if (rii>=h) {rii=h-1; riiw=0;}
+	if (ri>=h) {ri=h-1; riw=0;}
+	for(c=0; c<image->width; c++) {
+	    ci = c * mul / div;
+	    cii = (c+1) * mul / div;
+	    ciw = cii * div - c * mul;
+	    ciiw = (c+1) * mul - cii * div;
+	    if (cii>=w) {cii=w-1; ciiw=0;}
+	    if (ci>=w) {ci=w-1; ciw=0;}
+	    for (cl=0; cl<image->colors; cl++) {
+		iBuf[ri *w+ci ][cl] += image->image[r*wid+c][cl]*riw *ciw ;
+		iBuf[ri *w+cii][cl] += image->image[r*wid+c][cl]*riw *ciiw;
+		iBuf[rii*w+ci ][cl] += image->image[r*wid+c][cl]*riiw*ciw ;
+		iBuf[rii*w+cii][cl] += image->image[r*wid+c][cl]*riiw*ciiw;
+	    }
+	}
     }
     for (c=0; c<h*w; c++) for (cl=0; cl<image->colors; cl++)
-        image->image[c][cl] = iBuf[c][cl]/norm;
+	image->image[c][cl] = iBuf[c][cl]/norm;
     g_free(iBuf);
     image->height = h;
     image->width = w;
@@ -421,7 +466,7 @@ int dcraw_image_stretch(dcraw_image_data *image, double pixel_aspect)
 int dcraw_flip_image(dcraw_image_data *image, int flip)
 {
     if (flip)
-        flip_image_INDI(image->image, &image->height, &image->width, flip);
+	flip_image_INDI(image->image, &image->height, &image->width, flip);
     return DCRAW_SUCCESS;
 }
 
@@ -433,11 +478,11 @@ int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB)
     d->lastStatus = DCRAW_SUCCESS;
     memcpy(h->post_mul, h->pre_mul, sizeof h->post_mul);
     if (!d->is_foveon) /* foveon_interpolate() do this. */
-        /* BUG white should not be global */
-        scale_colors_INDI(h->raw.image,
-                h->rgbMax-h->black, h->black, useAutoWB, useCameraWB,
-                h->cam_mul, h->raw.height, h->raw.width, h->raw.colors,
-                h->post_mul, h->filters, d->white, d->ifname, d);
+	/* BUG white should not be global */
+	scale_colors_INDI(h->raw.image,
+		h->rgbMax-h->black, h->black, useAutoWB, useCameraWB,
+		h->cam_mul, h->raw.height, h->raw.width, h->raw.colors,
+		h->post_mul, h->filters, d->white, d->ifname, d);
     h->message = d->messageBuffer;
     return d->lastStatus;
 }
@@ -472,21 +517,24 @@ int dcraw_wavelet_denoise_shrinked(dcraw_image_data *f,
 }
 
 int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
-	int interpolation, int rgbWB[4])
+	dcraw_data *dark, int interpolation, int smoothing, int rgbWB[4])
 {
     DCRaw *d = (DCRaw *)h->dcraw;
-    int fujiWidth, i, r, c, cl;
+    int fujiWidth, i, r, c, cl, pixels;
     unsigned ff, f4;
 
     g_free(d->messageBuffer);
     d->messageBuffer = NULL;
     d->lastStatus = DCRAW_SUCCESS;
+    pixels = h->raw.width * h->raw.height;
 
     f->width = h->width;
     f->height = h->height;
     fujiWidth = h->fuji_width;
     f->colors = h->colors;
-    f->image = g_new0(dcraw_image_type, f->height * f->width);
+    f->image = (dcraw_image_type *)
+	g_realloc(f->image, f->height * f->width * sizeof(dcraw_image_type));
+    memset(f->image, 0, f->height * f->width * sizeof(dcraw_image_type));
 
     if (h->filters==0)
 	return DCRAW_ERROR;
@@ -494,42 +542,46 @@ int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
     cl = h->colors;
     if (interpolation==dcraw_four_color_interpolation || h->colors == 4) {
 	ff = h->fourColorFilters;
-        cl = 4;
+	cl = 4;
 	interpolation = dcraw_vng_interpolation;
     } else {
 	ff = h->filters &= ~((h->filters & 0x55555555) << 1);
     }
+    int black = dark ? MAX(h->black - dark->black, 0) : h->black;
+
     /* It might be better to report an error here: */
     /* (dcraw also forbids AHD for Fuji rotated images) */
-    if ((interpolation==dcraw_ahd_interpolation ||
-	 interpolation==dcraw_eahd_interpolation) && h->colors > 3)
+    if ( interpolation==dcraw_ahd_interpolation && h->colors > 3 )
 	interpolation = dcraw_vng_interpolation;
-    if (interpolation==dcraw_ppg_interpolation && h->colors > 3)
+    if ( interpolation==dcraw_ppg_interpolation && h->colors > 3 )
 	interpolation = dcraw_vng_interpolation;
     f4 = h->fourColorFilters;
     if (h->colors==3) rgbWB[3] = rgbWB[1];
     for(r=0; r<h->height; r++)
-        for(c=0; c<h->width; c++)
-            f->image[r*f->width+c][fc_INDI(ff,r,c)] = MIN( MAX( (gint64)
-                (h->raw.image[r/2*h->raw.width+c/2][fc_INDI(f4,r,c)] - h->black) *
-                rgbWB[fc_INDI(f4,r,c)]/0x10000, 0), 0xFFFF);
-
+	for(c=0; c<h->width; c++) {
+	    int cc = fc_INDI(f4,r,c);
+	    f->image[r*f->width+c][fc_INDI(ff,r,c)] = MIN( MAX( (gint64)
+		(get_pixel(h, dark, r/2, c/2, cc, pixels) - black) *
+		rgbWB[cc]/0x10000, 0), 0xFFFF);
+	}
+    int smoothPasses = 1;
     if (interpolation==dcraw_bilinear_interpolation)
 	lin_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
     else if (interpolation==dcraw_vng_interpolation)
 	vng_interpolate_INDI(f->image, ff, f->width, f->height, cl, 0xFFFF, d);
-    else if (interpolation==dcraw_ahd_interpolation)
+    else if (interpolation==dcraw_ahd_interpolation) {
 	ahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
 		h->rgb_cam, d);
-    else if (interpolation==dcraw_eahd_interpolation)
-	eahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
-		h->rgb_cam, d);
-    else if (interpolation==dcraw_ppg_interpolation)
+	smoothPasses = 3;
+    } else if (interpolation==dcraw_ppg_interpolation)
 	ppg_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
 
+    if ( smoothing )
+	color_smooth(f->image, f->width, f->height, smoothPasses);
+
     if (cl==4 && h->colors == 3) {
-        for (i=0; i<f->height*f->width; i++)
-            f->image[i][1] = (f->image[i][1]+f->image[i][3])/2;
+	for (i=0; i<f->height*f->width; i++)
+	    f->image[i][1] = (f->image[i][1]+f->image[i][3])/2;
     }
     fuji_rotate_INDI(&f->image, &f->height, &f->width, &fujiWidth,
 	    f->colors, h->fuji_step, d);
