@@ -20,6 +20,8 @@
 #endif
 
 #include <string.h>
+#include "uf_glib.h"
+#include "uf_gtk.h"
 #ifdef UFRAW_CINEPAINT
 /* Bypass a bug in CinePaint header files */
 #define RNH_FLOAT
@@ -46,7 +48,7 @@ void run(GIMP_CONST gchar *name,
 	gint *nreturn_vals,
 	GimpParam **return_vals);
 
-long ufraw_save_gimp_image(GtkWidget *widget, ufraw_data *uf);
+long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget);
 
 GimpPlugInInfo PLUG_IN_INFO = {
     NULL,  /* init_procedure */
@@ -59,32 +61,32 @@ MAIN()
 
 void query()
 {
-    static GimpParamDef load_args[] = {
+    static GIMP_CONST GimpParamDef load_args[] = {
 	{ GIMP_PDB_INT32,  "run_mode", "Interactive, non-interactive" },
 	{ GIMP_PDB_STRING, "filename", "The name of the file to load" },
 	{ GIMP_PDB_STRING, "raw_filename", "The name of the file to load" },
     };
-    static GimpParamDef load_return_vals[] = {
+    static GIMP_CONST GimpParamDef load_return_vals[] = {
 	{ GIMP_PDB_IMAGE, "image", "Output image" },
     };
 #if GIMP_CHECK_VERSION(2,2,0)
-    static GimpParamDef thumb_args[] = {
+    static GIMP_CONST GimpParamDef thumb_args[] = {
 	{ GIMP_PDB_STRING, "filename",     "The name of the file to load" },
 	{ GIMP_PDB_INT32,  "thumb_size",   "Preferred thumbnail size" }
     };
-    static GimpParamDef thumb_return_vals[] = {
+    static GIMP_CONST GimpParamDef thumb_return_vals[] = {
 	{ GIMP_PDB_IMAGE,  "image",        "Thumbnail image" },
 	{ GIMP_PDB_INT32,  "image_width",  "Width of full-sized image" },
 	{ GIMP_PDB_INT32,  "image_height", "Height of full-sized image" }
     };
 #endif
     gimp_install_procedure("file_ufraw_load",
-	    "Loads raw digital camera files",
-	    "This plug-in loads raw digital camera files.",
+	    "Loads digital camera raw files",
+	    "Loads digital camera raw files.",
 	    "Udi Fuchs",
 	    "Copyright 2003 by Dave Coffin\n"
 	    "Copyright 2004 by Pawel Jochym\n"
-	    "Copyright 2004 by Udi Fuchs",
+	    "Copyright 2004-2007 by Udi Fuchs",
 	    "ufraw-" VERSION,
 #if GIMP_CHECK_VERSION(2,2,0)
 	    "raw image",
@@ -99,12 +101,13 @@ void query()
 	    load_return_vals);
 
     gimp_register_load_handler("file_ufraw_load", (char *)raw_ext, "");
+
 #if GIMP_CHECK_VERSION(2,2,0)
     gimp_install_procedure ("file_ufraw_load_thumb",
-	    "Loads a small preview from a raw image",
-	    "",
-	    "",
-	    "Copyright 2004 by Udi Fuchs",
+	    "Loads thumbnails from digital camera raw files.",
+	    "Loads thumbnails from digital camera raw files.",
+	    "Udi Fuchs",
+	    "Copyright 2004-2007 by Udi Fuchs",
 	    "ufraw-" VERSION,
 	    NULL,
 	    NULL,
@@ -112,6 +115,7 @@ void query()
 	    G_N_ELEMENTS(thumb_args),
 	    G_N_ELEMENTS(thumb_return_vals),
 	    thumb_args, thumb_return_vals);
+
     gimp_register_thumbnail_loader("file_ufraw_load",
 	    "file_ufraw_load_thumb");
 #endif
@@ -128,12 +132,16 @@ char *gimp_get_progname()
 }
 #endif
 
+gboolean sendToGimpMode;
+
 void run(GIMP_CONST gchar *name,
 	gint nparams,
 	GIMP_CONST GimpParam *param,
 	gint *nreturn_vals,
 	GimpParam **return_vals)
 {
+    // TODO: Check if the static variable here is really needed.
+    // In any case this should cause no issues with threads.
     static GimpParam values[4];
     GimpRunMode run_mode;
     char *filename;
@@ -161,22 +169,22 @@ void run(GIMP_CONST gchar *name,
 	values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
 	return;
     }
-
+    gboolean loadThumbnail = size>0;
     gimp_ui_init("ufraw-gimp", TRUE);
 
     uf = ufraw_open(filename);
     /* if UFRaw fails on jpg or tif then open with GIMP */
     if (uf==NULL) {
 	if (!strcasecmp(filename + strlen(filename) - 4, ".jpg")) {
-	    if (size==0)
-		*return_vals = gimp_run_procedure2 ("file_jpeg_load",
+	    if ( loadThumbnail )
+		*return_vals = gimp_run_procedure2 ("file_jpeg_load_thumb",
 			nreturn_vals, nparams, param);
 	    else
-		*return_vals = gimp_run_procedure2 ("file_jpeg_load_thumb",
+		*return_vals = gimp_run_procedure2 ("file_jpeg_load",
 			nreturn_vals, nparams, param);
 	    return;
 	} else if (!strcasecmp(filename+strlen(filename)-4, ".tif")) {
-	    if (size==0)
+	    if ( !loadThumbnail )
 		*return_vals = gimp_run_procedure2 ("file_tiff_load",
 			nreturn_vals, nparams, param);
 	    else {
@@ -198,17 +206,11 @@ void run(GIMP_CONST gchar *name,
 	} else {
 	    /* Don't issue a message on thumbnail failure, since ufraw-gimp
 	     * will be called again with "file_ufraw_load" */
-	    if (size>0) return;
+	    if ( loadThumbnail ) return;
 
 	    ufraw_icons_init();
 	    GtkWidget *dummyWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#if GTK_CHECK_VERSION(2,6,0)
-	    gtk_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw");
-#else
-	    gtk_window_set_icon(GTK_WINDOW(dummyWindow),
-		    gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-			"ufraw", 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL));
-#endif
+	    uf_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw");
 	    ufraw_message(UFRAW_SET_PARENT, (char *)dummyWindow);
 
 	    ufraw_message(UFRAW_REPORT, NULL);
@@ -224,9 +226,9 @@ void run(GIMP_CONST gchar *name,
     conf_load(&conf, NULL);
 
     ufraw_config(uf, &conf, NULL, NULL);
-    conf_copy_save(uf->conf, &conf_default);
+    sendToGimpMode = (uf->conf->createID==send_id);
 #if GIMP_CHECK_VERSION(2,2,0)
-    if (size>0) {
+    if ( loadThumbnail ) {
 	uf->conf->size = size;
 	uf->conf->embeddedImage = TRUE;
     }
@@ -238,18 +240,34 @@ void run(GIMP_CONST gchar *name,
     values[0].type = GIMP_PDB_STATUS;
     values[0].data.d_status = GIMP_PDB_CANCEL;
     /* BUG - what should be done with GIMP_RUN_WITH_LAST_VALS */
-    if (size==0 && run_mode==GIMP_RUN_INTERACTIVE) {
-	status = ufraw_preview(uf, TRUE, ufraw_save_gimp_image);
-	gimp_set_data("plug_in_ufraw", &conf, sizeof(conf));
+    if ( run_mode==GIMP_RUN_INTERACTIVE &&
+	 !loadThumbnail && !sendToGimpMode ) {
+	/* Show the preview in interactive mode, unless if we are
+	 * in thumbnail mode or 'send to gimp' mode. */
+	status = ufraw_preview(uf, 1, ufraw_save_gimp_image);
     } else {
+	if ( sendToGimpMode ) {
+	    char *text = g_strdup_printf(_("Loading raw file '%s'"),
+		    uf->filename);
+	    gimp_progress_init(text);
+	    g_free(text);
+	}
+	if ( sendToGimpMode ) gimp_progress_update(0.1);
 	status=ufraw_load_raw(uf);
 	if ( status!=UFRAW_SUCCESS ) {
 	    values[0].type = GIMP_PDB_STATUS;
 	    values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	    return;
 	}
-	ufraw_save_gimp_image(NULL, uf);
+	if ( sendToGimpMode ) gimp_progress_update(0.3);
+	ufraw_save_gimp_image(uf, NULL);
+	if ( sendToGimpMode ) gimp_progress_update(1.0);
 	ufraw_close(uf);
+	/* To make sure we don't delete the raw file by mistake we check
+	 * that the file is really an ID file. */
+	if ( sendToGimpMode &&
+	     strcasecmp(filename + strlen(filename) - 6, ".ufraw")==0 )
+	    g_unlink(filename);
     }
     if ( status != UFRAW_SUCCESS || uf->gimpImage==-1 ) {
 	values[0].type = GIMP_PDB_STATUS;
@@ -264,7 +282,7 @@ void run(GIMP_CONST gchar *name,
     values[0].data.d_status = GIMP_PDB_SUCCESS;
     values[1].type = GIMP_PDB_IMAGE;
     values[1].data.d_image = uf->gimpImage;
-    if (size>0) {
+    if ( loadThumbnail ) {
 	*nreturn_vals = 4;
 	values[2].type = GIMP_PDB_INT32;
 	values[2].data.d_int32 = uf->initialWidth;
@@ -273,7 +291,7 @@ void run(GIMP_CONST gchar *name,
     }
 }
 
-long ufraw_save_gimp_image(GtkWidget *widget, ufraw_data *uf)
+long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 {
     GimpDrawable *drawable;
     GimpPixelRgn pixel_region;
@@ -296,10 +314,12 @@ long ufraw_save_gimp_image(GtkWidget *widget, ufraw_data *uf)
     } else {
 	if (ufraw_convert_image(uf)!=UFRAW_SUCCESS)
 	    return UFRAW_ERROR;
-	height = uf->conf->CropY2 - uf->conf->CropY1;
-	width = uf->conf->CropX2 - uf->conf->CropX1;
-	top = uf->conf->CropY1;
-	left = uf->conf->CropX1;
+	height = (uf->conf->CropY2 - uf->conf->CropY1)
+		* uf->image.height / uf->initialHeight;
+	width = (uf->conf->CropX2 - uf->conf->CropX1)
+		* uf->image.width / uf->initialWidth;
+	top = uf->conf->CropY1 * uf->image.height / uf->initialHeight;
+	left = uf->conf->CropX1 * uf->image.width / uf->initialWidth;
 #ifdef UFRAW_CINEPAINT
 	depth = 6;
 #else
@@ -338,8 +358,11 @@ long ufraw_save_gimp_image(GtkWidget *widget, ufraw_data *uf)
 	rowStride = uf->image.width;
 	rawImage = uf->image.image;
 	for (row = 0; row < height; row += tile_height) {
-	    preview_progress(widget, _("Loading image"),
-		    0.5 + 0.5*row/height);
+	    if ( sendToGimpMode )
+		gimp_progress_update(0.5 + 0.5*row/height);
+	    else
+		preview_progress(widget, _("Loading image"),
+			0.5 + 0.5*row/height);
 	    nrows = MIN(height-row, tile_height);
 	    for (y=0 ; y<nrows; y++)
 		develope(&pixbuf[y*width*depth], rawImage[(top+row+y)*rowStride+left],
