@@ -17,11 +17,11 @@
 #include <stdio.h>
 #include <math.h>
 #include <lcms.h>
-#include <glib.h>
+#include "uf_glib.h"
 #include "nikon_curve.h"
 #include "ufraw.h"
 
-int lcms_message(int ErrorCode, const char *ErrorText)
+static int lcms_message(int ErrorCode, const char *ErrorText)
 {
     /* Possible ErrorCode:
      * LCMS_ERRC_WARNING        0x1000
@@ -36,12 +36,13 @@ developer_data *developer_init()
 {
     int i;
     developer_data *d = g_new(developer_data,1);
+    d->mode = -1;
     d->gamma = -1;
     d->linear = -1;
     d->saturation = -1;
     for (i=0; i<profile_types; i++) {
-        d->profile[i] = NULL;
-        strcpy(d->profileFile[i],"no such file");
+	d->profile[i] = NULL;
+	strcpy(d->profileFile[i],"no such file");
     }
     memset(&d->baseCurveData, 0, sizeof(d->baseCurveData));
     d->baseCurveData.m_gamma = -1.0;
@@ -65,46 +66,86 @@ void developer_destroy(developer_data *d)
     int i;
     if (d==NULL) return;
     for (i=0; i<profile_types; i++)
-        if (d->profile[i]!=NULL) cmsCloseProfile(d->profile[i]);
+	if (d->profile[i]!=NULL) cmsCloseProfile(d->profile[i]);
     cmsCloseProfile(d->luminosityProfile);
     cmsFreeGamma(d->TransferFunction[0]);
     cmsFreeGamma(d->TransferFunction[1]);
     cmsCloseProfile(d->saturationProfile);
     if (d->colorTransform!=NULL)
-        cmsDeleteTransform(d->colorTransform);
+	cmsDeleteTransform(d->colorTransform);
     g_free(d);
 }
 
+static char *embedded_display_profile = "embedded display profile";
+
 /* Update the profile in the developer
  * and init values in the profile if needed */
 void developer_profile(developer_data *d, int type, profile_data *p)
 {
+    // embedded_display_profile were handled by developer_display_profile()
+    if ( strcmp(d->profileFile[type],embedded_display_profile)==0 )
+	return;
     if (strcmp(p->file, d->profileFile[type])) {
-        g_strlcpy(d->profileFile[type], p->file, max_path);
-        if (d->profile[type]!=NULL) cmsCloseProfile(d->profile[type]);
-        if (!strcmp(d->profileFile[type],""))
-            d->profile[type] = cmsCreate_sRGBProfile();
-        else {
-            d->profile[type] =
-                    cmsOpenProfileFromFile(d->profileFile[type], "r");
+	g_strlcpy(d->profileFile[type], p->file, max_path);
+	if (d->profile[type]!=NULL) cmsCloseProfile(d->profile[type]);
+	if (!strcmp(d->profileFile[type],""))
+	    d->profile[type] = cmsCreate_sRGBProfile();
+	else {
+	    char *filename =
+		    uf_win32_locale_filename_from_utf8(d->profileFile[type]);
+	    d->profile[type] = cmsOpenProfileFromFile(filename, "r");
+	    uf_win32_locale_filename_free(filename);
 	    if (d->profile[type]==NULL)
-                d->profile[type] = cmsCreate_sRGBProfile();
+		d->profile[type] = cmsCreate_sRGBProfile();
 	}
-        d->updateTransform = TRUE;
+	d->updateTransform = TRUE;
     }
     if (d->updateTransform) {
-        if (d->profile[type]!=NULL)
-            g_strlcpy(p->productName, cmsTakeProductName(d->profile[type]),
-                        max_name);
-        else
-            strcpy(p->productName, "");
+	if (d->profile[type]!=NULL)
+	    g_strlcpy(p->productName, cmsTakeProductName(d->profile[type]),
+		    max_name);
+	else
+	    strcpy(p->productName, "");
+    }
+}
+
+void developer_display_profile(developer_data *d,
+    unsigned char *profile, int size, char productName[])
+{
+    int type = display_profile;
+    if ( profile!=NULL ) {
+	if (d->profile[type]!=NULL) cmsCloseProfile(d->profile[type]);
+	d->profile[type] = cmsOpenProfileFromMem(profile, size);
+	g_free(profile);
+	if (d->profile[type]==NULL)
+	    d->profile[type] = cmsCreate_sRGBProfile();
+	if ( strcmp(d->profileFile[type], embedded_display_profile)!=0 ) {
+	    // start using embedded profile
+	    g_strlcpy(d->profileFile[type], embedded_display_profile, max_path);
+	    d->updateTransform = TRUE;
+	}
+    } else {
+	if ( strcmp(d->profileFile[type], embedded_display_profile)==0 ) {
+	    // embedded profile is no longer used
+	    if (d->profile[type]!=NULL) cmsCloseProfile(d->profile[type]);
+	    d->profile[type] = cmsCreate_sRGBProfile();
+	    strcpy(d->profileFile[type], "");
+	    d->updateTransform = TRUE;
+	}
+    }
+    if ( d->updateTransform ) {
+	if ( d->profile[type]!=NULL )
+	    g_strlcpy(productName, cmsTakeProductName(d->profile[type]),
+		    max_name);
+	else
+	    strcpy(productName, "");
     }
 }
 
 #define CLAMP_AB_DOUBLE(ab) \
 	( (ab<-128.0) ? -128.0 : ( (ab>127.9961) ? +127.9961 : ab) )
 
-int saturation_sampler(register WORD In[], register WORD Out[],
+static int saturation_sampler(register WORD In[], register WORD Out[],
 	register LPVOID Cargo)
 {
     cmsCIELab Lab;
@@ -127,7 +168,7 @@ int saturation_sampler(register WORD In[], register WORD Out[],
 }
 
 /* Based on lcms' cmsCreateBCHSWabstractProfile() */
-cmsHPROFILE create_saturation_profile(double saturation)
+static cmsHPROFILE create_saturation_profile(double saturation)
 {
     cmsHPROFILE hICC;
     LPLUT Lut;
@@ -144,10 +185,10 @@ cmsHPROFILE create_saturation_profile(double saturation)
     Lut = cmsAllocLUT();
     cmsAlloc3DGrid(Lut, 7, 3, 3);
     if (!cmsSample3DGrid(Lut, saturation_sampler, &saturation , 0)) {
-        // Shouldn't reach here
-        cmsFreeLUT(Lut);
-        cmsCloseProfile(hICC);
-        return NULL;
+	// Shouldn't reach here
+	cmsFreeLUT(Lut);
+	cmsCloseProfile(hICC);
+	return NULL;
     }
     // Create tags
     cmsAddTag(hICC, icSigMediaWhitePointTag, (LPVOID) cmsD50_XYZ());
@@ -159,7 +200,7 @@ cmsHPROFILE create_saturation_profile(double saturation)
 
 /* Find a for which (1-exp(-a x)/(1-exp(-a)) has derivative b at x=0 */
 /* In other words, solve a/(1-exp(-a))==b */
-double findExpCoeff(double b)
+static double findExpCoeff(double b)
 {
     double a, bg;
     int try;
@@ -174,34 +215,29 @@ double findExpCoeff(double b)
     return a;
 }
 
-void developer_create_transform(developer_data *d, DeveloperMode mode)
+static void developer_create_transform(developer_data *d, DeveloperMode mode)
 {
-    if ( mode==auto_developer ) {
-	/* For auto-tools we ignore all the output settings:
-	 * luminosity, saturation, output profile and proofing.
-	 * Instead we create an sRGB output. */
-	cmsHPROFILE *sRGBprofile = cmsCreate_sRGBProfile();
-	d->colorTransform = cmsCreateTransform(
-		d->profile[in_profile], TYPE_RGB_16,
-		sRGBprofile, TYPE_RGB_16, d->intent[out_profile], 0);
-        cmsCloseProfile(sRGBprofile);
+    if ( !d->updateTransform )
 	return;
-    }
+    d->updateTransform = FALSE;
+    if (d->colorTransform!=NULL)
+	cmsDeleteTransform(d->colorTransform);
+
     int targetProfile;
-    if ( mode==file_developer ) {
+    if ( mode==file_developer || mode==auto_developer ) {
 	targetProfile = out_profile;
     } else { /* mode==display_developer */
 	targetProfile = display_profile;
     }
     /* When softproofing is disabled, use the out_profile intent. */
-    if ( mode==file_developer ||
+    if ( mode==file_developer || mode==auto_developer ||
 	 d->intent[display_profile]==disable_intent ) {
 	/* No need for proofing transformation. */
-	if ( strcmp(d->profileFile[in_profile],"")==0 && 
+	if ( strcmp(d->profileFile[in_profile],"")==0 &&
 	     strcmp(d->profileFile[targetProfile],"")==0 &&
 	     d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
 	    /* No transformation at all. */
-            d->colorTransform = NULL;
+	    d->colorTransform = NULL;
 #if defined(LCMS_VERSION) && LCMS_VERSION <= 113 /* Bypass a lcms 1.13 bug. */
 	} else if ( d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
 	    d->colorTransform = cmsCreateTransform(
@@ -209,7 +245,7 @@ void developer_create_transform(developer_data *d, DeveloperMode mode)
 		    d->profile[targetProfile], TYPE_RGB_16,
 		    d->intent[out_profile], 0);
 #endif
-        } else {
+	} else {
 	    cmsHPROFILE prof[4];
 	    int i = 0;
 	    prof[i++] = d->profile[in_profile];
@@ -267,8 +303,16 @@ void developer_prepare(developer_data *d, conf_data *conf,
     profile_data *in, *out, *display;
     CurveData *baseCurve, *curve;
 
+    if ( mode!=d->mode ) {
+	d->mode = mode;
+	d->updateTransform = TRUE;
+    }
     in = &conf->profile[in_profile][conf->profileIndex[in_profile]];
-    out = &conf->profile[out_profile][conf->profileIndex[out_profile]];
+    /* For auto-tools we create an sRGB output. */
+    if ( mode==auto_developer )
+	out = &conf->profile[out_profile][0];
+    else
+	out = &conf->profile[out_profile][conf->profileIndex[out_profile]];
     display = &conf->profile[display_profile]
 		[conf->profileIndex[display_profile]];
     baseCurve = &conf->BaseCurve[conf->BaseCurveIndex];
@@ -292,7 +336,7 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	for (i=0; i<3; i++)
 	    for (c=0; c<d->colors; c++)
 		d->colorMatrix[i][c] = rgb_cam[i][c]*0x10000;
- 
+
     d->restoreDetails = conf->restoreDetails;
     int clipHighlights = conf->clipHighlights;
     unsigned exposure = pow(2, conf->exposure) * 0x10000;
@@ -316,8 +360,8 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	for (i=0; i<0x10000; i++) BaseCurve[i] = cs->m_Samples[i];
 	CurveSampleFree(cs);
 
-        d->gamma = in->gamma;
-        d->linear = in->linear;
+	d->gamma = in->gamma;
+	d->linear = in->linear;
 	d->exposure = exposure;
 	d->clipHighlights = clipHighlights;
 	guint16 FilmCurve[0x10000];
@@ -330,22 +374,22 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	} else { /* digital highlights */
 	    for (i=0; i<0x10000; i++) FilmCurve[i] = i;
 	}
-        double a, b, c, g;
+	double a, b, c, g;
 	/* The parameters of the linearized gamma curve are set in a way that
 	 * keeps the curve continuous and smooth at the connecting point.
 	 * d->linear also changes the real gamma used for the curve (g) in
 	 * a way that keeps the derivative at i=0x10000 constant.
 	 * This way changing the linearity changes the curve behaviour in
 	 * the shadows, but has a minimal effect on the rest of the range. */
-        if (d->linear<1.0) {
-            g = d->gamma*(1.0-d->linear)/(1.0-d->gamma*d->linear);
-            a = 1.0/(1.0+d->linear*(g-1));
-            b = d->linear*(g-1)*a;
-            c = pow(a*d->linear+b, g)/d->linear;
-        } else {
-            a = b = g = 0.0;
-            c = 1.0;
-        }
+	if (d->linear<1.0) {
+	    g = d->gamma*(1.0-d->linear)/(1.0-d->gamma*d->linear);
+	    a = 1.0/(1.0+d->linear*(g-1));
+	    b = d->linear*(g-1)*a;
+	    c = pow(a*d->linear+b, g)/d->linear;
+	} else {
+	    a = b = g = 0.0;
+	    c = 1.0;
+	}
 	for (i=0; i<0x10000; i++)
 	    if (BaseCurve[FilmCurve[i]]<0x10000*d->linear)
 		d->gammaCurve[i] = MIN(c*BaseCurve[FilmCurve[i]], 0xFFFF);
@@ -355,14 +399,20 @@ void developer_prepare(developer_data *d, conf_data *conf,
     }
     developer_profile(d, in_profile, in);
     developer_profile(d, out_profile, out);
-    developer_profile(d, display_profile, display);
     if ( conf->intent[out_profile]!=d->intent[out_profile] ) {
-        d->intent[out_profile] = conf->intent[out_profile];
-        d->updateTransform = TRUE;
+	d->intent[out_profile] = conf->intent[out_profile];
+	d->updateTransform = TRUE;
     }
+    /* For auto-tools we ignore all the output settings:
+     * luminosity, saturation, output profile and proofing. */
+    if ( mode==auto_developer ) {
+	developer_create_transform(d, mode);
+	return;
+    }
+    developer_profile(d, display_profile, display);
     if ( conf->intent[display_profile]!=d->intent[display_profile] ) {
-        d->intent[display_profile] = conf->intent[display_profile];
-        d->updateTransform = TRUE;
+	d->intent[display_profile] = conf->intent[display_profile];
+	d->updateTransform = TRUE;
     }
     /* Check if curve data has changed. */
     if (memcmp(curve, &d->luminosityCurveData, sizeof(CurveData))) {
@@ -399,23 +449,21 @@ void developer_prepare(developer_data *d, conf_data *conf,
 	    d->saturationProfile = create_saturation_profile(d->saturation);
 	d->updateTransform = TRUE;
     }
-    if (d->updateTransform || mode==auto_developer) {
-        if (d->colorTransform!=NULL)
-            cmsDeleteTransform(d->colorTransform);
-	developer_create_transform(d, mode);
-    }
+    developer_create_transform(d, mode);
 }
 
 extern const double xyz_rgb[3][3];
-const double rgb_xyz[3][3] = {			/* RGB from XYZ */
+static const double rgb_xyz[3][3] = {			/* RGB from XYZ */
     { 3.24048, -1.53715, -0.498536 },
     { -0.969255, 1.87599, 0.0415559 },
     { 0.0556466, -0.204041, 1.05731 } };
 
-void rgb_to_cielch(gint64 rgb[3], float lch[3])
+static void rgb_to_cielch(gint64 rgb[3], float lch[3])
 {
     int c, cc, i;
     float r, xyz[3], lab[3];
+    // The use of static varibles here should be thread safe.
+    // In the worst case cbrt[] will be calculated more than once.
     static gboolean firstRun = TRUE;
     static float cbrt[0x10000];
 
@@ -441,7 +489,7 @@ void rgb_to_cielch(gint64 rgb[3], float lch[3])
     lch[2] = atan2(lab[2], lab[1]);
 }
 
-void cielch_to_rgb(float lch[3], gint64 rgb[3])
+static void cielch_to_rgb(float lch[3], gint64 rgb[3])
 {
     int c, cc;
     float xyz[3], fx, fy, fz, xr, yr, zr, kappa, epsilon, tmpf, lab[3];
@@ -469,20 +517,20 @@ void cielch_to_rgb(float lch[3], gint64 rgb[3])
     }
 }
 
-void MaxMidMin(gint64 p[3], int *maxc, int *midc, int *minc)
+static void MaxMidMin(gint64 p[3], int *maxc, int *midc, int *minc)
 {
     if (p[0] > p[1] && p[0] > p[2]) {
-        *maxc = 0;
-        if (p[1] > p[2]) { *midc = 1; *minc = 2; }
-        else { *midc = 2; *minc = 1; }
+	*maxc = 0;
+	if (p[1] > p[2]) { *midc = 1; *minc = 2; }
+	else { *midc = 2; *minc = 1; }
     } else if (p[1] > p[2]) {
-        *maxc = 1;
-        if (p[0] > p[2]) { *midc = 0; *minc = 2; }
-        else { *midc = 2; *minc = 0; }
+	*maxc = 1;
+	if (p[0] > p[2]) { *midc = 0; *minc = 2; }
+	else { *midc = 2; *minc = 0; }
     } else {
-        *maxc = 2;
-        if (p[0] > p[1]) { *midc = 0; *minc = 1; }
-        else { *midc = 1; *minc = 0; }
+	*maxc = 2;
+	if (p[0] > p[1]) { *midc = 0; *minc = 1; }
+	else { *midc = 1; *minc = 0; }
     }
 }
 
@@ -498,7 +546,7 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
 	    buf[i*3+c] = d->gammaCurve[tmppix[c]];
     }
     if (d->colorTransform!=NULL)
-        cmsDoTransform(d->colorTransform, buf, buf, count);
+	cmsDoTransform(d->colorTransform, buf, buf, count);
 
     if (mode==16) for (i=0; i<3*count; i++) p16[i] = buf[i];
     else for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
