@@ -15,7 +15,7 @@
 #endif
 
 extern "C" {
-#include <glib.h>
+#include "uf_glib.h"
 #include "ufraw.h"
 }
 
@@ -41,16 +41,18 @@ try {
 	image = Exiv2::ImageFactory::open(
 		(const Exiv2::byte*)uf->unzippedBuf, uf->unzippedBufLen);
     } else {
-	image = Exiv2::ImageFactory::open(uf->filename);
+	char *filename = uf_win32_locale_filename_from_utf8(uf->filename);
+	image = Exiv2::ImageFactory::open(filename);
+	uf_win32_locale_filename_free(filename);
     }
     assert(image.get() != 0);
     image->readMetadata();
 
     Exiv2::ExifData &exifData = image->exifData();
     if (exifData.empty()) {
-        std::string error(uf->filename);
-        error += ": No Exif data found in the file";
-        throw Exiv2::Error(1, error);
+	std::string error(uf->filename);
+	error += ": No Exif data found in the file";
+	throw Exiv2::Error(1, error);
     }
     /* Reset orientation tag since UFRaw already rotates the image */
     Exiv2::ExifData::iterator pos;
@@ -198,6 +200,25 @@ try {
 	std::stringstream str;
        	str << *pos;
 	g_strlcpy(uf->conf->lensText, str.str().c_str(), max_name);
+    } else if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Canon.0x0095")))
+		!= exifData.end() ) {
+	std::stringstream str;
+	str << *pos;
+	g_strlcpy(uf->conf->lensText, str.str().c_str(), max_name);
+    }
+    /* Read flash mode */
+    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Photo.Flash")))
+	    != exifData.end() ) {
+	std::stringstream str;
+       	str << *pos;
+	g_strlcpy(uf->conf->flashText, str.str().c_str(), max_name);
+    }
+    /* Read White Balance Setting */
+    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Photo.WhiteBalance")))
+	    != exifData.end() ) {
+	std::stringstream str;
+	str << *pos;
+	g_strlcpy(uf->conf->whiteBalanceText, str.str().c_str(), max_name);
     }
     std::cerr.rdbuf(savecerr);
     ufraw_message(UFRAW_SET_LOG, "%s\n", stderror.str().c_str());
