@@ -15,36 +15,28 @@
 #endif
 
 #include <stdio.h>	/* for printf */
+#include <errno.h>
 #include <string.h>
-#include <math.h>	/* for fabs, floor */
-#include <gtk/gtk.h>
-#include <locale.h>
+#include "uf_glib.h"
 #include <glib/gi18n.h>
+#include "uf_gtk.h"
 #include "ufraw.h"
 
 /* Global information for the 'Save As' dialog */
 typedef struct {
     ufraw_data *uf;
-    double shrink;
-    double height;
-    double width;
-    gboolean FreezeDialog;
-    GtkAdjustment *shrinkAdj;
-    GtkAdjustment *heightAdj;
-    GtkAdjustment *widthAdj;
     GtkToggleButton *ppmButton;
     GtkToggleButton *tiffButton;
     GtkToggleButton *jpegButton;
+    GtkToggleButton *pngButton;
+    GtkToggleButton *fitsButton;
 } save_as_dialog_data;
 
-#define CROPPED_WIDTH (data->uf->conf->CropX2 - data->uf->conf->CropX1)
-#define CROPPED_HEIGHT (data->uf->conf->CropY2 - data->uf->conf->CropY1)
-
 #if GTK_CHECK_VERSION(2,6,0)
 void ufraw_chooser_toggle(GtkToggleButton *button, GtkFileChooser *filechooser);
 #endif
 
-void ufraw_radio_button_update(GtkWidget *button, int *valuep)
+static void ufraw_radio_button_update(GtkWidget *button, int *valuep)
 {
     GtkWidget *dialog = gtk_widget_get_ancestor(button, GTK_TYPE_DIALOG);
     if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
@@ -59,38 +51,12 @@ void ufraw_radio_button_update(GtkWidget *button, int *valuep)
     g_free(newname);
 }
 
-void ufraw_saver_adjustment_update(GtkAdjustment *adj,
-    save_as_dialog_data *data)
-{
-    if (data->FreezeDialog) return;
-    data->FreezeDialog = TRUE;
-
-    if ( adj==data->shrinkAdj ) {
-	data->shrink = gtk_adjustment_get_value(data->shrinkAdj);
-	data->height = CROPPED_HEIGHT / data->shrink;
-	data->width = CROPPED_WIDTH / data->shrink;
-    }
-    if ( adj==data->heightAdj ) {
-	data->height = gtk_adjustment_get_value(data->heightAdj);
-	data->width = data->height * CROPPED_WIDTH / CROPPED_HEIGHT;
-	data->shrink = (double)CROPPED_HEIGHT / data->height;
-    }
-    if ( adj==data->widthAdj ) {
-	data->width = gtk_adjustment_get_value(data->widthAdj);
-	data->height = data->width * CROPPED_HEIGHT / CROPPED_WIDTH;
-	data->shrink = (double)CROPPED_WIDTH / data->width;
-    }
-    gtk_adjustment_set_value(data->shrinkAdj, data->shrink);
-    gtk_adjustment_set_value(data->heightAdj, data->height);
-    gtk_adjustment_set_value(data->widthAdj, data->width);
-
-    data->FreezeDialog = FALSE;
-}
-
-void ufraw_saver_set_type(GtkWidget *widget, save_as_dialog_data *data)
+static void ufraw_saver_set_type(GtkWidget *widget, save_as_dialog_data *data)
 {
     char *filename, *type;
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+    if (filename==NULL)
+	return;
     type = strrchr(filename, '.');
     if (type==NULL) {
 	g_free(filename);
@@ -111,11 +77,23 @@ void ufraw_saver_set_type(GtkWidget *widget, save_as_dialog_data *data)
 	data->uf->conf->type = jpeg_type;
 	gtk_toggle_button_set_active(data->jpegButton, TRUE);
     }
+#endif
+#ifdef HAVE_LIBPNG
+    if ( !strcmp(type,".png") ) {
+	data->uf->conf->type = png8_type;
+	gtk_toggle_button_set_active(data->pngButton, TRUE);
+    }
+#endif
+#ifdef HAVE_LIBCFITSIO
+    if ( !strcmp(type,".fits") ) {
+	data->uf->conf->type = fits_type;
+	gtk_toggle_button_set_active(data->fitsButton, TRUE);
+    }
 #endif
     g_free(filename);
 }
 
-int ufraw_overwrite_dialog(char *filename, GtkWidget *widget)
+static int ufraw_overwrite_dialog(char *filename, GtkWidget *widget)
 {
     char message[max_path];
     int response;
@@ -138,7 +116,7 @@ int ufraw_overwrite_dialog(char *filename, GtkWidget *widget)
     return (response==GTK_RESPONSE_YES);
 }
 
-long ufraw_saver(void *widget, gpointer user_data)
+long ufraw_save_as(ufraw_data *uf, void *widget)
 {
     GtkFileChooser *fileChooser;
     GtkWidget *expander, *box, *table, *widg, *button, *align, *overwriteButton;
@@ -147,75 +125,17 @@ long ufraw_saver(void *widget, gpointer user_data)
 #if defined(HAVE_LIBZ) && defined(HAVE_LIBTIFF)
     GtkWidget *losslessButton;
 #endif
-#ifdef HAVE_LIBJPEG
-#if defined(HAVE_LIBEXIF) || defined(HAVE_EXIV2)
+#if defined(HAVE_EXIV2) && (defined(HAVE_LIBJPEG) || defined(HAVE_LIBPNG))
     GtkWidget *exifButton;
 #endif
+#ifdef HAVE_LIBJPEG
     GtkAdjustment *compressAdj;
+    GtkWidget *progressiveButton;
 #endif
-    ufraw_data *uf = user_data;
-    char *filename, *absFilename;
     int status;
     save_as_dialog_data DialogData, *data = &DialogData;
     data->uf = uf;
-    data->FreezeDialog = TRUE;
 
-    if (uf->conf->size > 0) {
-	if (CROPPED_HEIGHT > CROPPED_WIDTH) {
-	    data->height = uf->conf->size;
-	    data->width = uf->conf->size * CROPPED_WIDTH / CROPPED_HEIGHT;
-	    data->shrink = (double)CROPPED_HEIGHT / uf->conf->size;
-	} else {
-	    data->width = uf->conf->size;
-	    data->height = uf->conf->size * CROPPED_HEIGHT / CROPPED_WIDTH;
-	    data->shrink = (double)CROPPED_WIDTH / uf->conf->size;
-	}
-    } else {
-	if (uf->conf->shrink<1) {
-	    ufraw_message(UFRAW_ERROR, _("Fatal Error: uf->conf->shrink<1"));
-	    uf->conf->shrink = 1;
-	}
-	data->height = CROPPED_HEIGHT / uf->conf->shrink;
-	data->width = CROPPED_WIDTH / uf->conf->shrink;
-	data->shrink = uf->conf->shrink;
-    }
-    if ( strlen(uf->conf->outputFilename)>0 )
-	filename = uf_file_set_type(uf->conf->outputFilename,
-		file_type[uf->conf->type]);
-    else
-	filename = uf_file_set_type(uf->filename, file_type[uf->conf->type]);
-    if (strlen(uf->conf->outputPath)>0) {
-	char *cp = g_path_get_basename(filename);
-	g_free(filename);
-	filename = g_build_filename(uf->conf->outputPath, cp, NULL);
-	g_free(cp);
-    }
-    absFilename = uf_file_set_absolute(filename);
-    if (widget==NULL) {
-	/* Function was only called to collect data. */
-	char *utf8 = g_filename_to_utf8(absFilename, -1, NULL, NULL, NULL);
-	if (utf8==NULL) utf8 = g_strdup("Unknown file name");
-	char *text = g_strdup_printf(_("Filename: %s\nSize: %d x %d%s"),
-		utf8, (int)data->width, (int)data->height,
-		uf->conf->createID==also_id ? _("\nCreate also ID file") :
-		uf->conf->createID==only_id ? _("\nCreate only ID file") : "");
-	g_free(utf8);
-	g_free(absFilename);
-	g_free(filename);
-	return (long)text;
-    }
-    if (!strcmp(gtk_button_get_label(GTK_BUTTON(widget)), GTK_STOCK_SAVE)) {
-	if ( !uf->conf->overwrite && uf->conf->createID!=only_id
-	   && g_file_test(absFilename, G_FILE_TEST_EXISTS) ) {
-	    if ( !ufraw_overwrite_dialog(filename, widget) )
-		return UFRAW_ERROR;
-	}
-	g_strlcpy(uf->conf->outputFilename, absFilename, max_path);
-	g_free(filename);
-	g_free(absFilename);
-	status = ufraw_write_image(uf);
-	return status;
-    }
     fileChooser = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(
 	    _("Save image"), GTK_WINDOW(gtk_widget_get_toplevel(widget)),
 	    GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -232,21 +152,15 @@ long ufraw_saver(void *widget, gpointer user_data)
 	    G_CALLBACK(ufraw_chooser_toggle), fileChooser);
     gtk_file_chooser_set_extra_widget(fileChooser, hidden_button);
 #endif
+    char *absFilename = uf_file_set_absolute(uf->conf->outputFilename);
     char *path = g_path_get_dirname(absFilename);
     gtk_file_chooser_set_current_folder(fileChooser, path);
     g_free(path);
-    char *base = g_path_get_basename(filename);
+    g_free(absFilename);
+    char *base = g_path_get_basename(uf->conf->outputFilename);
     gtk_file_chooser_set_current_name(fileChooser, base);
     g_free(base);
 
-    GtkTooltips *tips = gtk_tooltips_new();
-#if GTK_CHECK_VERSION(2,10,0)
-    g_object_ref_sink(GTK_OBJECT(tips));
-#else
-    g_object_ref(tips);
-    gtk_object_sink(GTK_OBJECT(tips));
-#endif
-
     expander = gtk_expander_new(_("Output options"));
     gtk_expander_set_expanded(GTK_EXPANDER(expander), TRUE);
     gtk_file_chooser_set_extra_widget(fileChooser, expander);
@@ -259,35 +173,6 @@ long ufraw_saver(void *widget, gpointer user_data)
 	ufraw_message(UFRAW_WARNING, _("Interpolation method set to AHD"));
     }
 
-    table = gtk_table_new(5, 1, FALSE);
-    gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);
-    widg = gtk_label_new(_("Shrink factor "));
-    gtk_table_attach(GTK_TABLE(table), widg, 0, 1, 1, 2, 0, 0, 0, 0);
-    data->shrinkAdj = GTK_ADJUSTMENT(gtk_adjustment_new(data->shrink,
-	    1, 100, 1, 2, 3));
-    g_signal_connect(G_OBJECT(data->shrinkAdj), "value-changed",
-	    G_CALLBACK(ufraw_saver_adjustment_update), data);
-    widg = gtk_spin_button_new(data->shrinkAdj, 1, 3);
-    gtk_table_attach(GTK_TABLE(table), widg, 1, 2, 1, 2, 0, 0, 0, 0);
-
-    widg = gtk_label_new(_("\tHeight "));
-    gtk_table_attach(GTK_TABLE(table), widg, 2, 3, 1, 2, 0, 0, 0, 0);
-    data->heightAdj = GTK_ADJUSTMENT(gtk_adjustment_new(data->height,
-	    CROPPED_HEIGHT/100, CROPPED_HEIGHT, 10, 100, 0));
-    g_signal_connect(G_OBJECT(data->heightAdj), "value-changed",
-	    G_CALLBACK(ufraw_saver_adjustment_update), data);
-    widg = gtk_spin_button_new(data->heightAdj, 10, 0);
-    gtk_table_attach(GTK_TABLE(table), widg, 3, 4, 1, 2, 0, 0, 0, 0);
-
-    widg = gtk_label_new(_("\tWidth "));
-    gtk_table_attach(GTK_TABLE(table), widg, 4, 5, 1, 2, 0, 0, 0, 0);
-    data->widthAdj = GTK_ADJUSTMENT(gtk_adjustment_new(data->width,
-	    CROPPED_WIDTH/100, CROPPED_WIDTH, 10, 100, 0));
-    g_signal_connect(G_OBJECT(data->widthAdj), "value-changed",
-	    G_CALLBACK(ufraw_saver_adjustment_update), data);
-    widg = gtk_spin_button_new(data->widthAdj, 10, 0);
-    gtk_table_attach(GTK_TABLE(table), widg, 5, 6, 1, 2, 0, 0, 0, 0);
-
     gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
     button = gtk_radio_button_new_with_label(NULL, _("8-bit ppm"));
     data->ppmButton = GTK_TOGGLE_BUTTON(button);
@@ -306,6 +191,7 @@ long ufraw_saver(void *widget, gpointer user_data)
     g_signal_connect(G_OBJECT(button), "toggled",
 	    G_CALLBACK(ufraw_radio_button_update), &uf->conf->type);
     gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
+
 #ifdef HAVE_LIBTIFF
     gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
     table = gtk_table_new(5, 1, FALSE);
@@ -338,7 +224,7 @@ long ufraw_saver(void *widget, gpointer user_data)
 #endif /*HAVE_LIBTIFF*/
 #ifdef HAVE_LIBJPEG
     gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
-    table = gtk_table_new(5, 1, FALSE);
+    table = gtk_table_new(5, 2, FALSE);
     gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);
     button = gtk_radio_button_new_with_label_from_widget(
 	    GTK_RADIO_BUTTON(button), "JPEG");
@@ -359,14 +245,58 @@ long ufraw_saver(void *widget, gpointer user_data)
 	    GTK_EXPAND|GTK_FILL, 0, 0, 0);
     widg = gtk_spin_button_new(compressAdj, 5, 0);
     gtk_table_attach(GTK_TABLE(table), widg, 3, 4, 1, 2, 0, 0, 0, 0);
-#if defined(HAVE_LIBEXIF) || defined(HAVE_EXIV2)
-    exifButton = gtk_check_button_new_with_label(_("Embed EXIF data"));
+    progressiveButton = gtk_check_button_new_with_label(_("Progressive encoding"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(progressiveButton),
+    				 uf->conf->progressiveJPEG);
+    gtk_table_attach(GTK_TABLE(table), progressiveButton, 1, 2, 2, 3, 0, 0, 0, 0);
+#endif /*HAVE_LIBJPEG*/
+#ifdef HAVE_LIBPNG
+    gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
+    table = gtk_table_new(5, 1, FALSE);
+    gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);
+    button = gtk_radio_button_new_with_label_from_widget(
+	    GTK_RADIO_BUTTON(button), _("8-bit PNG"));
+    data->pngButton = GTK_TOGGLE_BUTTON(button);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+	    uf->conf->type==png8_type);
+    g_object_set_data(G_OBJECT(button), "ButtonValue", (gpointer)png8_type);
+    g_signal_connect(G_OBJECT(button), "toggled",
+	    G_CALLBACK(ufraw_radio_button_update), &uf->conf->type);
+    align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
+    gtk_container_add(GTK_CONTAINER(align), button);
+    gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL,0, 0, 0);
+    button = gtk_radio_button_new_with_label_from_widget(
+	    GTK_RADIO_BUTTON(button), _("16-bit PNG"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+	    uf->conf->type==png16_type);
+    g_object_set_data(G_OBJECT(button), "ButtonValue", (gpointer)png16_type);
+    g_signal_connect(G_OBJECT(button), "toggled",
+	    G_CALLBACK(ufraw_radio_button_update), &uf->conf->type);
+    gtk_table_attach(GTK_TABLE(table), button, 0, 1, 1, 2, 0, 0, 0, 0);
+#endif /*HAVE_LIBPNG*/
+#ifdef HAVE_LIBCFITSIO
+    gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
+
+    button = gtk_radio_button_new_with_label_from_widget(
+	    GTK_RADIO_BUTTON(button), _("Flexible Image Transport System (FITS)"));
+    data->fitsButton = GTK_TOGGLE_BUTTON(button);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+	    uf->conf->type==fits_type);
+    g_object_set_data(G_OBJECT(button), "ButtonValue", (gpointer)fits_type);
+    g_signal_connect(G_OBJECT(button), "toggled",
+	    G_CALLBACK(ufraw_radio_button_update), &uf->conf->type);
+    gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
+#endif
+
+#if defined(HAVE_EXIV2) && (defined(HAVE_LIBJPEG) || defined(HAVE_LIBPNG))
+    gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
+    exifButton = gtk_check_button_new_with_label(
+	    _("Embed EXIF data in JPEG or PNG files"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(exifButton),
 	    uf->conf->embedExif && uf->exifBuf!=NULL);
     gtk_widget_set_sensitive(exifButton, uf->exifBuf!=NULL);
-    gtk_table_attach(GTK_TABLE(table), exifButton, 1, 2, 2, 3, 0, 0, 0, 0);
-#endif
-#endif /*HAVE_LIBJPEG*/
+    gtk_box_pack_start(GTK_BOX(box), exifButton, TRUE, TRUE, 0);
+#endif /*HAVE_EXIV2 && (HAVE_LIBJPEG || HAVE_LIBPNG)*/
 
     gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), TRUE, TRUE, 0);
     table = gtk_table_new(1, 5, FALSE);
@@ -384,9 +314,9 @@ long ufraw_saver(void *widget, gpointer user_data)
     label = gtk_label_new(_("\tSave image defaults "));
     event = gtk_event_box_new();
     gtk_container_add(GTK_CONTAINER(event), label);
-    gtk_tooltips_set_tip(tips, event,
+    uf_widget_set_tooltip(event,
 	    _("Save current image manipulation parameters as defaults.\n"
-	    "The output parameters in this window are always saved."), NULL);
+	    "The output parameters in this window are always saved."));
     gtk_table_attach(GTK_TABLE(table), event, 3, 4, 0, 1, 0, 0, 0, 0);
     confCombo = GTK_COMBO_BOX(gtk_combo_box_new_text());
     gtk_combo_box_append_text(confCombo, _("Never again"));
@@ -412,7 +342,6 @@ long ufraw_saver(void *widget, gpointer user_data)
     }
     g_signal_connect(G_OBJECT(fileChooser), "selection-changed",
 	    G_CALLBACK(ufraw_saver_set_type), data);
-    data->FreezeDialog = FALSE;
     while(1) {
 	status = gtk_dialog_run(GTK_DIALOG(fileChooser));
 	if (status==GTK_RESPONSE_CANCEL) {
@@ -421,16 +350,9 @@ long ufraw_saver(void *widget, gpointer user_data)
 	    return UFRAW_CANCEL;
 	}
 	/* GTK_RESPONSE_OK - save file */
-	filename = gtk_file_chooser_get_filename(fileChooser);
+	char *filename = gtk_file_chooser_get_filename(fileChooser);
 	uf->conf->createID = gtk_combo_box_get_active(idCombo);
 	uf->conf->saveConfiguration = gtk_combo_box_get_active(confCombo);
-	if ( fabs(data->shrink-floor(data->shrink+0.0005))<0.0005 ) {
-	    uf->conf->shrink = floor(data->shrink+0.0005);
-	    uf->conf->size = 0;
-	} else {
-	    uf->conf->shrink = 1;
-	    uf->conf->size = floor(MAX(data->height, data->width)+0.5);
-	}
 	uf->conf->overwrite = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(overwriteButton));
 #if defined(HAVE_LIBZ) && defined(HAVE_LIBTIFF)
@@ -439,10 +361,12 @@ long ufraw_saver(void *widget, gpointer user_data)
 #endif
 #ifdef HAVE_LIBJPEG
 	uf->conf->compression = gtk_adjustment_get_value(compressAdj);
-#if defined(HAVE_LIBEXIF) || defined(HAVE_EXIV2)
+	uf->conf->progressiveJPEG = gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(progressiveButton));
+#endif
+#if defined(HAVE_EXIV2) && (defined(HAVE_LIBJPEG) || defined(HAVE_LIBPNG))
 	uf->conf->embedExif = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(exifButton));
-#endif
 #endif
 	if ( !uf->conf->overwrite && uf->conf->createID!=only_id
 	   && g_file_test(filename, G_FILE_TEST_EXISTS) ) {
@@ -452,10 +376,102 @@ long ufraw_saver(void *widget, gpointer user_data)
 	g_strlcpy(uf->conf->outputFilename, filename, max_path);
 	g_free(filename);
 	status = ufraw_write_image(uf);
-	if (status==UFRAW_SUCCESS) {
+	if ( status==UFRAW_ERROR ) {
+	    ufraw_message(status, ufraw_get_message(uf));
+	} else {
+	    if ( status==UFRAW_WARNING )
+		ufraw_message(status, ufraw_get_message(uf));
+	    if ( ufraw_get_message(uf)!=NULL )
+		ufraw_message(UFRAW_SET_LOG, ufraw_get_message(uf));
 	    ufraw_focus(fileChooser, FALSE);
 	    gtk_widget_destroy(GTK_WIDGET(fileChooser));
 	    return UFRAW_SUCCESS;
 	}
     }
 }
+
+long ufraw_save_now(ufraw_data *uf, void *widget)
+{
+    if ( !uf->conf->overwrite && uf->conf->createID!=only_id
+	&& g_file_test(uf->conf->outputFilename, G_FILE_TEST_EXISTS) ) {
+	if ( !ufraw_overwrite_dialog(uf->conf->outputFilename, widget) )
+	    return UFRAW_ERROR;
+    }
+    int status = ufraw_write_image(uf);
+    if ( status==UFRAW_ERROR ) {
+	ufraw_message(status, ufraw_get_message(uf));
+	return UFRAW_ERROR;
+    }
+    if ( ufraw_get_message(uf)!=NULL )
+	ufraw_message(UFRAW_SET_LOG, ufraw_get_message(uf));
+    return UFRAW_SUCCESS;
+}
+
+long ufraw_send_to_gimp(ufraw_data *uf)
+{
+    char *basename = g_path_get_basename(uf->conf->inputFilename);
+    char *template = g_strconcat(basename, "_XXXXXX", NULL);
+    g_free(basename);
+    char *confFilename = NULL;
+    GError *err = NULL;
+    int fd = g_file_open_tmp(template, &confFilename, &err);
+    g_free(template);
+    if ( fd==-1 ) {
+	g_free(confFilename);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error creating temporary file."), err->message);
+	g_error_free(err);
+	return UFRAW_ERROR;
+    }
+    FILE *out = fdopen(fd, "w");
+    if ( out==NULL ) {
+	g_free(confFilename);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error creating temporary file."), g_strerror(errno));
+	return UFRAW_ERROR;
+    }
+    char *buffer;
+    int saveCreateID = uf->conf->createID;
+    uf->conf->createID = send_id;
+    conf_save(uf->conf, confFilename, &buffer);
+    uf->conf->createID = saveCreateID;
+    if ( fwrite(buffer, strlen(buffer), 1, out)!=1 ) {
+	g_free(buffer);
+	g_free(confFilename);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error creating temporary file."), g_strerror(errno));
+	return UFRAW_ERROR;
+    }
+    g_free(buffer);
+    if ( fclose(out)!=0 ) {
+	g_free(confFilename);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error creating temporary file."), g_strerror(errno));
+	return UFRAW_ERROR;
+    }
+    char *fullConfFilename = g_strconcat(confFilename, ".ufraw", NULL);
+    if ( g_rename(confFilename, fullConfFilename)==-1 ) {
+	g_free(confFilename);
+	g_free(fullConfFilename);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error creating temporary file."), g_strerror(errno));
+	return UFRAW_ERROR;
+    }
+    g_free(confFilename);
+    char *commandLine = g_strdup_printf("%s \"%s\"",
+	    uf->conf->remoteGimpCommand, fullConfFilename);
+    g_free(fullConfFilename);
+    /* gimp-remote starts the gimp in a fork().
+     * Therefore we must call it asynchronously. */
+    if ( !g_spawn_command_line_async(commandLine, &err) ) {
+	g_free(commandLine);
+	ufraw_message(UFRAW_ERROR, "%s\n%s",
+		_("Error activating Gimp."), err->message);
+	g_error_free(err);
+	return UFRAW_ERROR;
+    }
+    g_free(commandLine);
+    // Sleep for 0.2 seconds, giving time for gimp-remote to do the X query.
+    g_usleep(200*1000);
+    return UFRAW_SUCCESS;
+}
