@@ -14,12 +14,9 @@
 #include "config.h"
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h> /* for fstat() */
 #include <math.h>
-#include <time.h>
 #include <errno.h>
 #ifdef HAVE_LIBZ
 #include <zlib.h>
@@ -27,98 +24,15 @@
 #ifdef HAVE_LIBBZ2
 #include <bzlib.h>
 #endif
-#include <glib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include "uf_glib.h"
 #include <glib/gi18n.h>
 #include "dcraw_api.h"
 #include "nikon_curve.h"
 #include "ufraw.h"
 
-char *ufraw_message_buffer(char *buffer, char *message)
-{
-#ifdef UFRAW_DEBUG
-    ufraw_batch_messenger(message);
-#endif
-    char *buf;
-    if (buffer==NULL) return g_strdup(message);
-    buf = g_strconcat(buffer, message, NULL);
-    g_free(buffer);
-    return buf;
-}
-
-void ufraw_batch_messenger(char *message)
-{
-    /* Print the 'ufraw:' header only if there are no newlines in the message
-     * (not including possibly one at the end).
-     * Otherwise, the header will be printed only for the first line. */
-    if (g_strstr_len(message, strlen(message)-1, "\n")==NULL)
-	g_printerr("%s: ", ufraw_binary);
-    g_printerr("%s%c", message, message[strlen(message)-1]!='\n' ? '\n' : 0);
-}
-
-char *ufraw_message(int code, const char *format, ...)
-{
-    static char *logBuffer = NULL;
-    static char *errorBuffer = NULL;
-    static gboolean errorFlag = FALSE;
-    static void *parentWindow = NULL;
-    char *message = NULL;
-    void *saveParentWindow;
-
-    if (code==UFRAW_SET_PARENT) {
-        saveParentWindow = parentWindow;
-        parentWindow = (void *)format;
-        return saveParentWindow;
-    }
-    if (format!=NULL) {
-        va_list ap;
-        va_start(ap, format);
-        message = g_strdup_vprintf (format, ap);
-        va_end(ap);
-    }
-    switch (code) {
-    case UFRAW_SET_ERROR:
-                errorFlag = TRUE;
-    case UFRAW_SET_WARNING:
-                errorBuffer = ufraw_message_buffer(errorBuffer, message);
-    case UFRAW_SET_LOG:
-    case UFRAW_DCRAW_SET_LOG:
-                logBuffer = ufraw_message_buffer(logBuffer, message);
-                g_free(message);
-                return NULL;
-    case UFRAW_GET_ERROR:
-                if (!errorFlag) return NULL;
-    case UFRAW_GET_WARNING:
-                return errorBuffer;
-    case UFRAW_GET_LOG:
-                return logBuffer;
-    case UFRAW_CLEAN:
-                g_free(logBuffer);
-                logBuffer = NULL;
-    case UFRAW_RESET:
-                g_free(errorBuffer);
-                errorBuffer = NULL;
-                errorFlag = FALSE;
-                return NULL;
-    case UFRAW_BATCH_MESSAGE:
-                if (parentWindow==NULL)
-                    ufraw_messenger(message, parentWindow);
-                g_free(message);
-                return NULL;
-    case UFRAW_INTERACTIVE_MESSAGE:
-                if (parentWindow!=NULL)
-		    ufraw_messenger(message, parentWindow);
-                g_free(message);
-                return NULL;
-    case UFRAW_REPORT:
-                ufraw_messenger(errorBuffer, parentWindow);
-                return NULL;
-    default:
-                ufraw_messenger(message, parentWindow);
-                g_free(message);
-                return NULL;
-    }
-}
-
 static int make_temporary(char *basefilename, char **tmpfilename)
 {
     int fd;
@@ -143,38 +57,42 @@ static ssize_t writeall(int fd, const char *buf, ssize_t size)
 static char *decompress_gz(char *origfilename)
 {
 #ifdef HAVE_LIBZ
-    char *filename;
+    char *tempfilename;
     int tmpfd;
     gzFile gzfile;
     char buf[8192];
     ssize_t size;
-    if ((tmpfd = make_temporary(origfilename, &filename)) != -1) {
-	if ((gzfile = gzopen(origfilename, "rb")) != 0) {
-	    while ((size = gzread(gzfile, buf, sizeof buf)) > 0) {
-		if (writeall(tmpfd, buf, size) != size)
-		    break;
-	    }
-	    gzclose(gzfile);
-	    if (size == 0)
-		if (close(tmpfd) == 0)
-		    return filename;
+    if ( (tmpfd = make_temporary(origfilename, &tempfilename))==-1 )
+	return NULL;
+    char *filename = uf_win32_locale_filename_from_utf8(origfilename);
+    gzfile = gzopen(filename, "rb");
+    uf_win32_locale_filename_free(filename);
+    if ( gzfile!=NULL ) {
+	while ((size = gzread(gzfile, buf, sizeof buf)) > 0) {
+	    if (writeall(tmpfd, buf, size) != size)
+		break;
 	}
-	close(tmpfd);
-	unlink(filename);
-	free(filename);
+	gzclose(gzfile);
+	if (size == 0)
+	    if (close(tmpfd) == 0)
+		return tempfilename;
     }
+    close(tmpfd);
+    g_unlink(tempfilename);
+    g_free(tempfilename);
+    return NULL;
 #else
-    origfilename = origfilename;
+    (void)origfilename;
     ufraw_message(UFRAW_SET_ERROR,
 		  "Cannot open gzip compressed images.\n");
-#endif
     return NULL;
+#endif
 }
 
 static char *decompress_bz2(char *origfilename)
 {
 #ifdef HAVE_LIBBZ2
-    char *filename;
+    char *tempfilename;
     int tmpfd;
     FILE *compfile;
     BZFILE *bzfile;
@@ -182,32 +100,32 @@ static char *decompress_bz2(char *origfilename)
     char buf[8192];
     ssize_t size;
 
-    if ((tmpfd = make_temporary(origfilename, &filename)) != -1) {
-	if ((compfile = fopen(origfilename, "rb")) != 0) {
-	    if ((bzfile = BZ2_bzReadOpen(&bzerror, compfile,
-					 0, 0, 0, 0)) != 0) {
-	        while ((size = BZ2_bzRead(&bzerror, bzfile,
-					  buf, sizeof buf)) > 0)
-		    if (writeall(tmpfd, buf, size) != size)
-		        break;
-		BZ2_bzReadClose(&bzerror, bzfile);
-		fclose(compfile);
-		if (size == 0) {
-		    close(tmpfd);
-		    return filename;
-		}
+    if ( (tmpfd = make_temporary(origfilename, &tempfilename))==-1 )
+	return NULL;
+    compfile = g_fopen(origfilename, "rb");
+    if ( compfile!=NULL ) {
+	if ((bzfile = BZ2_bzReadOpen(&bzerror, compfile, 0, 0, 0, 0)) != 0) {
+	    while ((size = BZ2_bzRead(&bzerror, bzfile, buf, sizeof buf)) > 0)
+		if (writeall(tmpfd, buf, size) != size)
+		    break;
+	    BZ2_bzReadClose(&bzerror, bzfile);
+	    fclose(compfile);
+	    if (size == 0) {
+		close(tmpfd);
+		return tempfilename;
 	    }
 	}
-	close(tmpfd);
-	unlink(filename);
-	free(filename);
     }
+    close(tmpfd);
+    g_unlink(tempfilename);
+    g_free(tempfilename);
+    return NULL;
 #else
-    origfilename = origfilename;
+    (void)origfilename;
     ufraw_message(UFRAW_SET_ERROR,
 		  "Cannot open bzip2 compressed images.\n");
-#endif
     return NULL;
+#endif
 }
 
 ufraw_data *ufraw_open(char *filename)
@@ -233,8 +151,8 @@ ufraw_data *ufraw_open(char *filename)
 	g_strlcpy(filename, fname, max_path);
 	g_free(fname);
     }
-    /* First handle ufraw ID files */
-    if (!strcasecmp(filename + strlen(filename) - 6, ".ufraw")) {
+    /* First handle ufraw ID files. */
+    if ( strcasecmp(filename + strlen(filename) - 6, ".ufraw")==0 ) {
 	conf = g_new(conf_data, 1);
 	status = conf_load(conf, filename);
 	if (status!=UFRAW_SUCCESS) {
@@ -251,7 +169,7 @@ ufraw_data *ufraw_open(char *filename)
 	if ( strcmp(inPath, outPath)==0 ) {
 	    char *path = g_path_get_dirname(filename);
 	    char *inName = g_path_get_basename(conf->inputFilename);
-	    char *inFile = g_strconcat(path, "/", inName, NULL);
+	    char *inFile = g_build_filename(path, inName , NULL);
 	    if ( g_file_test(inFile, G_FILE_TEST_EXISTS) ) {
 		g_strlcpy(conf->inputFilename, inFile, max_path);
 	    }
@@ -283,13 +201,13 @@ ufraw_data *ufraw_open(char *filename)
     status = dcraw_open(raw, filename);
     if (filename != origfilename) {
 	g_file_get_contents(filename, &unzippedBuf, &unzippedBufLen, NULL);
-	unlink(filename);
-	free(filename);
+	g_unlink(filename);
+	g_free(filename);
 	filename = origfilename;
     }
     if ( status!=DCRAW_SUCCESS) {
-        /* Hold the message without displaying it */
-        ufraw_message(UFRAW_SET_WARNING, raw->message);
+	/* Hold the message without displaying it */
+	ufraw_message(UFRAW_SET_WARNING, raw->message);
 	if ( status!=DCRAW_WARNING) {
 	    g_free(raw);
 	    g_free(unzippedBuf);
@@ -297,6 +215,7 @@ ufraw_data *ufraw_open(char *filename)
 	}
     }
     uf = g_new0(ufraw_data, 1);
+    ufraw_message_init(uf);
     uf->unzippedBuf = unzippedBuf;
     uf->unzippedBufLen = unzippedBufLen;
     uf->conf = conf;
@@ -309,42 +228,89 @@ ufraw_data *ufraw_open(char *filename)
     uf->raw = raw;
     uf->colors = raw->colors;
     uf->raw_color = raw->raw_color;
-    uf->developer = developer_init();
+    uf->developer = NULL;
+    uf->AutoDeveloper = NULL;
     uf->widget = NULL;
     uf->RawLumHistogram = NULL;
     uf->HaveFilters = raw->filters!=0;
     ufraw_message(UFRAW_SET_LOG, "ufraw_open: w:%d h:%d curvesize:%d\n",
-        uf->initialWidth, uf->initialHeight, raw->toneCurveSize);
- 
+	    raw->width, raw->height, raw->toneCurveSize);
+
     return uf;
 }
 
-ufraw_data *ufraw_load_darkframe(char *darkframeFile)
+int ufraw_load_darkframe(ufraw_data *uf)
 {
-    ufraw_data *uf;
-    if (strlen(darkframeFile)>0) {
-        uf = ufraw_open(darkframeFile);
-        if (!uf){
-            ufraw_message(UFRAW_ERROR,
-		    _("darkframe error: %s is not a raw file\n"),
-		    darkframeFile);
-            return NULL;
-        }
-	uf->conf = g_new(conf_data, 1);
-	*uf->conf = conf_default;
-	/* disable all auto settings on darkframe */
-	uf->conf->autoExposure = disabled_state;
-	uf->conf->autoBlack = disabled_state;
-        if (ufraw_load_raw(uf)==UFRAW_SUCCESS){
-            ufraw_message(UFRAW_BATCH_MESSAGE, _("using darkframe '%s'\n"),
-		    uf->filename);
-            return uf;
-        }
-        else
-            ufraw_message(UFRAW_ERROR, _("error loading darkframe '%s'\n"),
-		    uf->filename);
+    if ( strlen(uf->conf->darkframeFile)==0 )
+	return UFRAW_SUCCESS;
+    if ( uf->conf->darkframe!=NULL ) {
+	// If the same file was already openned, there is nothing to do.
+	if ( strcmp(uf->conf->darkframeFile, uf->conf->darkframe->filename)==0 )
+	    return UFRAW_SUCCESS;
+	// Otherwise we need to close the previous darkframe
+	ufraw_close(uf->conf->darkframe);
     }
-    return NULL;
+    ufraw_data *dark = uf->conf->darkframe =
+	    ufraw_open(uf->conf->darkframeFile);
+    if ( dark==NULL ){
+	ufraw_message(UFRAW_ERROR, _("darkframe error: %s is not a raw file\n"),
+		    uf->conf->darkframeFile);
+	uf->conf->darkframeFile[0] = '\0';
+	return UFRAW_ERROR;
+    }
+    dark->conf = g_new(conf_data, 1);
+    *dark->conf = conf_default;
+    /* disable all auto settings on darkframe */
+    dark->conf->autoExposure = disabled_state;
+    dark->conf->autoBlack = disabled_state;
+    if (ufraw_load_raw(dark)!=UFRAW_SUCCESS) {
+	ufraw_message(UFRAW_ERROR, _("error loading darkframe '%s'\n"),
+		uf->conf->darkframeFile);
+	ufraw_close(dark);
+	g_free(dark);
+	uf->conf->darkframe = NULL;
+	uf->conf->darkframeFile[0] = '\0';
+	return UFRAW_ERROR;
+    }
+    // Make sure the darkframe matches the main data
+    dcraw_data *raw = uf->raw;
+    dcraw_data *darkRaw = dark->raw;
+    if ( raw->width!=darkRaw->width ||
+	 raw->height!=darkRaw->height ||
+	 raw->colors!=darkRaw->colors ) {
+	ufraw_message(UFRAW_WARNING,
+		_("Darkframe '%s' is incompatible with main image"),
+		uf->conf->darkframeFile);
+	ufraw_close(dark);
+	g_free(dark);
+	uf->conf->darkframe = NULL;
+	uf->conf->darkframeFile[0] = '\0';
+	return UFRAW_ERROR;
+    }
+    ufraw_message(UFRAW_BATCH_MESSAGE, _("using darkframe '%s'\n"),
+	    uf->conf->darkframeFile);
+    /* Calculate dark frame hot pixel thresholds as the 99.99th percentile
+     * value.  That is, the value at which 99.99% of the pixels are darker.
+     * Pixels below this threshold are considered to be bias noise, and
+     * those above are "hot". */
+    int color;
+    int i;
+    long frequency[65536];
+    long sum;
+    long point = darkRaw->raw.width * darkRaw->raw.height / 10000;
+
+    for (color = 0; color < darkRaw->raw.colors; ++color) {
+	memset(frequency, 0, sizeof frequency);
+	for (i = 0; i < darkRaw->raw.width * darkRaw->raw.height; ++i)
+	    frequency[darkRaw->raw.image[i][color]]++;
+	for (sum = 0, i = 65535; i > 1; --i) {
+	    sum += frequency[i];
+	    if (sum >= point)
+		break;
+	}
+	darkRaw->thresholds[color] = i + 1;
+    }
+    return UFRAW_SUCCESS;
 }
 
 int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
@@ -402,6 +368,19 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	uf->conf->CropX2 = -1;
 	uf->conf->CropY2 = -1;
     }
+    if (strlen(uf->conf->outputFilename)==0) {
+	/* If output filename wasn't specified use input filename */
+	char *filename = uf_file_set_type(uf->filename,
+			file_type[uf->conf->type]);
+	if (strlen(uf->conf->outputPath)>0) {
+	    char *cp = g_path_get_basename(filename);
+	    g_free(filename);
+	    filename = g_build_filename(uf->conf->outputPath, cp , NULL);
+	    g_free(cp);
+	}
+	g_strlcpy(uf->conf->outputFilename, filename, max_path);
+	g_free(filename);
+    }
     /*Reset EXIF data text fields to avoid spill over between images.*/
     strcpy(uf->conf->isoText, "");
     strcpy(uf->conf->shutterText, "");
@@ -409,9 +388,12 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     strcpy(uf->conf->focalLenText, "");
     strcpy(uf->conf->focalLen35Text, "");
     strcpy(uf->conf->lensText, "");
-    if (ufraw_exif_from_exiv2(uf)!=UFRAW_SUCCESS) {
-        ufraw_message(UFRAW_SET_LOG, "Error reading EXIF data from %s\n",
-                uf->filename);
+    strcpy(uf->conf->flashText, "");
+    if ( !uf->conf->embeddedImage ) {
+	if ( ufraw_exif_from_exiv2(uf)!=UFRAW_SUCCESS ) {
+	    ufraw_message(UFRAW_SET_LOG, "Error reading EXIF data from %s\n",
+		    uf->filename);
+	}
     }
     g_free(uf->unzippedBuf);
     uf->unzippedBuf = NULL;
@@ -426,17 +408,26 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     }
     /* Set the EXIF data */
 #ifdef __MINGW32__
-    /* For MinG32 we don't have the thread safe ctime_r */
-    g_strlcpy(uf->conf->timestamp, ctime(&raw->timestamp), max_name);
-#elif defined(SOLARIS)
-    ctime_r(&raw->timestamp, uf->conf->timestamp, sizeof(uf->conf->timestamp));
+    /* MinG32 does not have ctime_r(). */
+    g_strlcpy(uf->conf->timestampText, ctime(&raw->timestamp), max_name);
+#elif defined(__sun) /* Solaris */
+    /*
+     * Some (all?) versions of Solaris followed a draft POSIX.1c standard
+     * where ctime_r took a third length argument.
+     */
+    ctime_r(&raw->timestamp, uf->conf->timestampText,
+	    sizeof(uf->conf->timestampText));
 #else
-    ctime_r(&raw->timestamp, uf->conf->timestamp);
+    /* POSIX.1c version of ctime_r() */
+    ctime_r(&raw->timestamp, uf->conf->timestampText);
 #endif
-    if (uf->conf->timestamp[strlen(uf->conf->timestamp)-1]=='\n')
-	uf->conf->timestamp[strlen(uf->conf->timestamp)-1] = '\0';
+    if (uf->conf->timestampText[strlen(uf->conf->timestampText)-1]=='\n')
+	uf->conf->timestampText[strlen(uf->conf->timestampText)-1] = '\0';
     g_strlcpy(uf->conf->make, raw->make, max_name);
     g_strlcpy(uf->conf->model, raw->model, max_name);
+
+    uf->conf->timestamp = raw->timestamp;
+
     if ( !uf->LoadingID || uf->conf->orientation<0 )
 	uf->conf->orientation = raw->flip;
     if (uf->exifBuf==NULL) {
@@ -512,12 +503,7 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 		uf->conf->BaseCurveIndex==camera_curve)
 	    uf->conf->BaseCurveIndex = linear_curve;
     }
-    /* Using DarkframeFile from the configuration is disabled since we still
-     * cannot open two raw files simultaniously */
-    /*
-    if (strcmp(uf->conf->darkframeFile, cmd->darkframeFile)!=0)
-	uf->conf->darkframe = ufraw_load_darkframe(uf->conf->darkframeFile);
-    */
+    ufraw_load_darkframe(uf);
 
     dcraw_image_dimensions(raw, uf->conf->orientation,
 	    &uf->initialHeight, &uf->initialWidth);
@@ -530,36 +516,6 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     return UFRAW_SUCCESS;
 }
 
-void ufraw_substract_darkframe(ufraw_data *uf)
-{
-    dcraw_data *df = uf->conf->darkframe->raw;
-    dcraw_data *org = uf->raw;
-    int i, cl;
-
-    if (org->raw.width!=df->raw.width &&
-        org->raw.width!=df->raw.width &&
-        org->raw.colors!=df->raw.colors){
-
-        ufraw_message(UFRAW_SET_WARNING,
-		_("Darkframe is incompatible with main image"));
-        return;
-    }
-
-    for( i=0; i<org->raw.height*org->raw.width; i++ ) {
-        for( cl=0; cl<org->raw.colors; cl++ ){
-            org->raw.image[i][cl] =
-		    org->raw.image[i][cl] >= df->raw.image[i][cl] ?
-                    org->raw.image[i][cl] - df->raw.image[i][cl] :
-                    0;
-        }
-    }
-    org->black = org->black >= df->black ?
-	    org->black - df->black :
-	    0;
-    ufraw_message(UFRAW_SET_LOG, "subtracted darkframe '%s'\n",
-	    uf->conf->darkframeFile);
-}
-
 int ufraw_load_raw(ufraw_data *uf)
 {
     int status;
@@ -577,16 +533,9 @@ int ufraw_load_raw(ufraw_data *uf)
     }
     if ( (status=dcraw_load_raw(raw))!=DCRAW_SUCCESS ) {
 	ufraw_message(UFRAW_SET_LOG, raw->message);
-        ufraw_message(status, raw->message);
-        if (status!=DCRAW_WARNING) return status;
+	ufraw_message(status, raw->message);
+	if (status!=DCRAW_WARNING) return status;
     }
-    /* raw->black can change in ufraw_subtract_darkframe() so we must
-     * calculate uf->rgbMax first */
-    uf->rgbMax = raw->rgbMax - raw->black;
-    if (uf->conf->darkframe!=NULL)
-        ufraw_substract_darkframe(uf);
-    memcpy(uf->rgb_cam, raw->rgb_cam, sizeof uf->rgb_cam);
-
     /* Canon EOS cameras require special exposure normalization */
     if ( strcmp(uf->conf->make, "Canon")==0 &&
 	 strncmp(uf->conf->model, "EOS", 3)==0 ) {
@@ -603,16 +552,19 @@ int ufraw_load_raw(ufraw_data *uf)
 	    /* Convert exposure value from old ID files from before
 	     * ExposureNorm */
 	    if (uf->LoadingID && uf->conf->ExposureNorm==0)
-		uf->conf->exposure -= log(1.0*uf->rgbMax/max)/log(2);
-	    uf->conf->ExposureNorm = max;
+		uf->conf->exposure -= log(1.0*raw->rgbMax/max)/log(2);
+	    uf->conf->ExposureNorm = max * raw->rgbMax / 4095;
 	    ufraw_message(UFRAW_SET_LOG,
 		    "Exposure Normalization set to %d (%.2f EV)\n",
 		    uf->conf->ExposureNorm,
-		    log(1.0*uf->rgbMax/uf->conf->ExposureNorm)/log(2));
+		    log(1.0*raw->rgbMax/uf->conf->ExposureNorm)/log(2));
 	}
     } else {
 	uf->conf->ExposureNorm = 0;
     }
+    uf->rgbMax = raw->rgbMax - raw->black;
+    memcpy(uf->rgb_cam, raw->rgb_cam, sizeof uf->rgb_cam);
+
     /* Foveon image dimensions are knows only after load_raw()*/
     dcraw_image_dimensions(raw, uf->conf->orientation,
 	    &uf->initialHeight, &uf->initialWidth);
@@ -652,16 +604,35 @@ void ufraw_close(ufraw_data *uf)
 	g_free(uf->Images[i].buffer);
     g_free(uf->thumb.buffer);
     developer_destroy(uf->developer);
+    developer_destroy(uf->AutoDeveloper);
     g_free(uf->RawLumHistogram);
+    if ( uf->conf->darkframe!=NULL ) {
+	ufraw_close(uf->conf->darkframe);
+	g_free(uf->conf->darkframe);
+    }
+    ufraw_message_reset(uf);
     ufraw_message(UFRAW_CLEAN, NULL);
 }
 
+void ufraw_developer_prepare(ufraw_data *uf, DeveloperMode mode)
+{
+    if ( mode==auto_developer ) {
+	if ( uf->AutoDeveloper==NULL )
+	    uf->AutoDeveloper = developer_init();
+	developer_prepare(uf->AutoDeveloper, uf->conf,
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, mode);
+    } else {
+	if ( uf->developer==NULL )
+	    uf->developer = developer_init();
+	developer_prepare(uf->developer, uf->conf,
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, mode);
+    }
+}
+
 int ufraw_convert_image_init(ufraw_data *uf)
 {
     dcraw_data *raw = uf->raw;
 
-    developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, file_developer);
     /* We can do a simple interpolation in the following cases:
      * We shrink by an integer value.
      * If pixel_aspect<1 (e.g. NIKON D1X) shrink must be at least 4.
@@ -669,8 +640,8 @@ int ufraw_convert_image_init(ufraw_data *uf)
      * There are no filters (Foveon). */
     uf->ConvertShrink = 1;
     if ( uf->conf->interpolation==half_interpolation ||
-         ( uf->conf->size==0 && uf->conf->shrink>1 ) ||
-         ( uf->conf->size>0 &&
+	 ( uf->conf->size==0 && uf->conf->shrink>1 ) ||
+	 ( uf->conf->size>0 &&
 	   uf->conf->size<=MAX(raw->raw.height, raw->raw.width) ) ||
 	 !uf->HaveFilters ) {
 	if (uf->conf->size==0 && uf->conf->shrink>1 &&
@@ -686,14 +657,15 @@ int ufraw_convert_image_init(ufraw_data *uf)
 
 int ufraw_convert_image(ufraw_data *uf)
 {
+    ufraw_developer_prepare(uf, file_developer);
     ufraw_convert_image_init(uf);
     ufraw_convert_image_first_phase(uf);
     if (uf->ConvertShrink>1) {
 	dcraw_data *raw = uf->raw;
 	dcraw_image_data final;
-        final.height = uf->image.height;
-        final.width = uf->image.width;
-        final.image = uf->image.image;
+	final.height = uf->image.height;
+	final.width = uf->image.width;
+	final.image = uf->image.image;
 	/* Scale threshold according to shrink factor, as the effect of
 	 * neighbouring pixels decays about exponentially with distance. */
 	float threshold = uf->conf->threshold * exp(-(uf->ConvertShrink/2.0-1));
@@ -709,25 +681,21 @@ int ufraw_convert_image_first_phase(ufraw_data *uf)
     int status, c;
     dcraw_data *raw = uf->raw;
     dcraw_image_data final;
+    // final.image memory will be realloc'd as needed
+    final.image = uf->image.image;
+    dcraw_data *dark = uf->conf->darkframe ? uf->conf->darkframe->raw : NULL;
 
     if ( uf->ConvertShrink>1 || !uf->HaveFilters ) {
-        dcraw_finalize_shrink(&final, raw, uf->ConvertShrink);
-
-        uf->image.height = final.height;
-        uf->image.width = final.width;
-	g_free(uf->image.image);
-        uf->image.image = final.image;
+	dcraw_finalize_shrink(&final, raw, dark, uf->ConvertShrink);
     } else {
 	if ( (status=dcraw_wavelet_denoise(raw,
 		uf->conf->threshold))!=DCRAW_SUCCESS )
 	    return status;
-        dcraw_finalize_interpolate(&final, raw, uf->conf->interpolation,
-		uf->developer->rgbWB);
+	dcraw_finalize_interpolate(&final, raw, dark, uf->conf->interpolation,
+		uf->conf->smoothing, uf->developer->rgbWB);
 	uf->developer->rgbMax = uf->developer->max;
-        for (c=0; c<4; c++)
+	for (c=0; c<4; c++)
 	    uf->developer->rgbWB[c] = 0x10000;
-	g_free(uf->image.image);
-        uf->image.image = final.image;
     }
     dcraw_image_stretch(&final, raw->pixel_aspect);
     if (uf->conf->size==0 && uf->conf->shrink>1) {
@@ -736,16 +704,23 @@ int ufraw_convert_image_first_phase(ufraw_data *uf)
 	uf->ConvertShrink = uf->conf->shrink;
     }
     if (uf->conf->size>0) {
-        if ( uf->conf->size>MAX(final.height, final.width) ) {
-            ufraw_message(UFRAW_ERROR, _("Can not downsize from %d to %d."),
-                    MAX(final.height, final.width), uf->conf->size);
-        } else {
-	    uf->ConvertShrink = MAX(final.height, final.width)/uf->conf->size;
-            dcraw_image_resize(&final, uf->conf->size);
+	int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
+	int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
+	int cropSize = MAX(cropHeight, cropWidth);
+	if ( uf->conf->size > cropSize ) {
+	    ufraw_message(UFRAW_ERROR, _("Can not downsize from %d to %d."),
+		    cropSize, uf->conf->size);
+	} else {
+	    /* uf->conf->size holds the size of the cropped image.
+	     * We need to calculate from it the desired size of
+	     * th uncropped image. */
+	    int finalSize = uf->ConvertShrink * MAX(final.height, final.width);
+	    uf->ConvertShrink = cropSize / uf->conf->size;
+	    dcraw_image_resize(&final, uf->conf->size * finalSize / cropSize);
 	}
     }
-    uf->image.image = final.image;
     dcraw_flip_image(&final, uf->conf->orientation);
+    uf->image.image = final.image;
     uf->image.height = final.height;
     uf->image.width = final.width;
 
@@ -831,7 +806,7 @@ int ufraw_convert_image_area(ufraw_data *uf,
     return UFRAW_SUCCESS;
 }
 
-int ufraw_flip_image_buffer(ufraw_image_data *img, int flip)
+static int ufraw_flip_image_buffer(ufraw_image_data *img, int flip)
 {
     /* Following code was copied from dcraw's flip_image()
      * and modified to work with any pixel depth. */
@@ -895,7 +870,7 @@ int ufraw_flip_image(ufraw_data *uf, int flip)
     if ( uf->conf->threshold>0 )
 	ufraw_flip_image_buffer(&uf->Images[ufraw_denoise_phase], flip);
     ufraw_flip_image_buffer(&uf->Images[ufraw_final_phase], flip);
- 
+
     return UFRAW_SUCCESS;
 }
 
@@ -968,34 +943,34 @@ int ufraw_set_wb(ufraw_data *uf)
 		}
 	    }
 	}
-        for (c=0; c<uf->colors; c++) {
+	for (c=0; c<uf->colors; c++) {
 	    gint64 sum = 0;
-            for (i=0; i<uf->rgbMax+1; i++)
-                sum += (gint64)i*histogram[i][c];
+	    for (i=0; i<uf->rgbMax+1; i++)
+		sum += (gint64)i*histogram[i][c];
 	    if (sum==0) uf->conf->chanMul[c] = 1.0;
-            else uf->conf->chanMul[c] = 1.0/sum;
-        }
+	    else uf->conf->chanMul[c] = 1.0/sum;
+	}
 	g_free(histogram);
 	uf->conf->WBTuning = 0;
     } else if ( !strcmp(uf->conf->wb, camera_wb) ) {
-        if ( (status=dcraw_set_color_scale(raw,
+	if ( (status=dcraw_set_color_scale(raw,
 		!strcmp(uf->conf->wb, auto_wb),
 		!strcmp(uf->conf->wb, camera_wb)))!=DCRAW_SUCCESS ) {
-            if (status==DCRAW_NO_CAMERA_WB) {
-                ufraw_message(UFRAW_BATCH_MESSAGE,
-                    _("Cannot use camera white balance, "
-                    "reverting to auto white balance."));
-                ufraw_message(UFRAW_INTERACTIVE_MESSAGE,
-                    _("Cannot use camera white balance, "
-                    "reverting to auto white balance."));
+	    if (status==DCRAW_NO_CAMERA_WB) {
+		ufraw_message(UFRAW_BATCH_MESSAGE,
+		    _("Cannot use camera white balance, "
+		    "reverting to auto white balance."));
+		ufraw_message(UFRAW_INTERACTIVE_MESSAGE,
+		    _("Cannot use camera white balance, "
+		    "reverting to auto white balance."));
 		g_strlcpy(uf->conf->wb, auto_wb, max_name);
 		return ufraw_set_wb(uf);
-            }
-            if (status!=DCRAW_SUCCESS)
+	    }
+	    if (status!=DCRAW_SUCCESS)
 		return status;
-        }
-        for (c=0; c<raw->colors; c++)
-            uf->conf->chanMul[c] = raw->post_mul[c];
+	}
+	for (c=0; c<raw->colors; c++)
+	    uf->conf->chanMul[c] = raw->post_mul[c];
 	uf->conf->WBTuning = 0;
     } else {
 	int lastTuning = -1;
@@ -1091,7 +1066,7 @@ int ufraw_set_wb(ufraw_data *uf)
     return UFRAW_SUCCESS;
 }
 
-void ufraw_build_raw_luminosity_histogram(ufraw_data *uf)
+static void ufraw_build_raw_luminosity_histogram(ufraw_data *uf)
 {
     int i, c;
     gint64 max;
@@ -1117,11 +1092,11 @@ void ufraw_build_raw_luminosity_histogram(ufraw_data *uf)
     memset(uf->RawLumHistogram, 0, (uf->rgbMax+1)*sizeof(int));
     uf->RawLumCount = raw->raw.height*raw->raw.width;
     for (i=0; i<uf->RawLumCount; i++) {
-        for (c=0, max=0; c<raw->raw.colors; c++) {
-            max = MAX( (gint64)(raw->raw.image[i][c]-raw->black) *
+	for (c=0, max=0; c<raw->raw.colors; c++) {
+	    max = MAX( (gint64)(raw->raw.image[i][c]-raw->black) *
 			uf->RawLumChanMul[c], max);
-        }
-        uf->RawLumHistogram[MIN(max/0x10000,uf->rgbMax)]++;
+	}
+	uf->RawLumHistogram[MIN(max/0x10000,uf->rgbMax)]++;
     }
 }
 
@@ -1138,8 +1113,7 @@ void ufraw_auto_expose(ufraw_data *uf)
     /* If we normalize the exposure then 0 EV also gets normalized */
     if ( uf->conf->ExposureNorm>0 )
 	uf->conf->exposure = -log(1.0*uf->rgbMax/uf->conf->ExposureNorm)/log(2);
-    developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
+    ufraw_developer_prepare(uf, auto_developer);
     /* Find the grey value that gives 99% luminosity */
     double maxChan = 0;
     for (c=0; c<uf->colors; c++) maxChan = MAX(uf->conf->chanMul[c], maxChan);
@@ -1147,7 +1121,7 @@ void ufraw_auto_expose(ufraw_data *uf)
     {
 	for (c=0; c<uf->colors; c++)
 	    pix[c] = MIN (p * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-	develope(p16, pix, uf->developer, 16, pixtmp, 1);
+	develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
 	for (c=0, wp=0; c<3; c++) wp = MAX(wp, p16[c]);
 	if (wp < 0x10000 * 99/100) pMin = p;
 	else pMax = p;
@@ -1157,7 +1131,7 @@ void ufraw_auto_expose(ufraw_data *uf)
     stop = uf->RawLumCount * 1/100;
     /* Calculate the white point */
     for (wp=uf->rgbMax, sum=0; wp>1 && sum<stop; wp--)
-                sum += uf->RawLumHistogram[wp];
+	sum += uf->RawLumHistogram[wp];
     /* Set 99% of the luminosity values with luminosity below 99% */
     uf->conf->exposure = log((double)p/wp)/log(2);
     /* If we are going to normalize the exposure later,
@@ -1179,22 +1153,21 @@ void ufraw_auto_black(ufraw_data *uf)
     if (uf->conf->autoBlack==disabled_state) return;
 
     /* Reset the luminosityCurve */
-    developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
+    ufraw_developer_prepare(uf, auto_developer);
     /* Calculate the black point */
     ufraw_build_raw_luminosity_histogram(uf);
     stop = uf->RawLumCount/256/4;
     for (bp=0, sum=0; bp<uf->rgbMax && sum<stop; bp++)
-                sum += uf->RawLumHistogram[bp];
+	sum += uf->RawLumHistogram[bp];
     double maxChan = 0;
     for (c=0; c<uf->colors; c++) maxChan = MAX(uf->conf->chanMul[c], maxChan);
     for (c=0; c<uf->colors; c++)
 	pix[c] = MIN (bp * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-    develope(p16, pix, uf->developer, 16, pixtmp, 1);
+    develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
     for (c=0, bp=0; c<3; c++) bp = MAX(bp, p16[c]);
 
     CurveDataSetPoint(&uf->conf->curve[uf->conf->curveIndex],
-            0, (double)bp/0x10000, 0);
+	    0, (double)bp/0x10000, 0);
 
     uf->conf->autoBlack = enabled_state;
 //    ufraw_message(UFRAW_SET_LOG, "ufraw_auto_black: "
@@ -1214,8 +1187,7 @@ void ufraw_auto_curve(ufraw_data *uf)
     double norm = (1-pow(decay,steps))/(1-decay);
 
     CurveDataReset(curve);
-    developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
+    ufraw_developer_prepare(uf, auto_developer);
     /* Calculate curve points */
     ufraw_build_raw_luminosity_histogram(uf);
     stop = uf->RawLumCount/256/4;
@@ -1223,10 +1195,10 @@ void ufraw_auto_curve(ufraw_data *uf)
     for (c=0; c<uf->colors; c++) maxChan = MAX(uf->conf->chanMul[c], maxChan);
     for (bp=0, sum=0, p=0, i=j=0; i<steps && bp<uf->rgbMax && p<0xFFFF; i++) {
 	for (; bp<uf->rgbMax && sum<stop; bp++)
-            sum += uf->RawLumHistogram[bp];
+	    sum += uf->RawLumHistogram[bp];
 	for (c=0; c<uf->colors; c++)
 	    pix[c] = MIN (bp * maxChan/uf->conf->chanMul[c], uf->rgbMax);
-	develope(p16, pix, uf->developer, 16, pixtmp, 1);
+	develope(p16, pix, uf->AutoDeveloper, 16, pixtmp, 1);
 	for (c=0, p=0; c<3; c++) p = MAX(p, p16[c]);
 	stop += uf->RawLumCount * pow(decay,i) / norm;
 	/* Skeep adding point if slope is too big (more than 4) */
